//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************

#include "bufmgr.h"
#include "debug.h"

bool BufMgr::add(QString fname, FILE_FORMAT format) {
	if (mindex_ >= (MAXBUFMGR - 1))
		return false;
	data_[mindex_].fname_ = fname;
	data_[mindex_].format_ = format;
	data_[mindex_].active_ = true;
	data_[mindex_].text_ = "";
	data_[mindex_].line_ = 1;
	data_[mindex_].col_ = 1;
	aindex_ = mindex_;
	mindex_++;
	return true;
}

bool BufMgr::del(int index) {
	if (index >= mindex_)
		return false;
	// some cleanup on active_ first
	for (int i = 0; i < mindex_; i++) {
		if (data_[i].active_ == false)
			data_[i].active_ = true;
	}

	data_[index].active_ = false;
	data_[index].fname_ = "";
	data_[index].text_ = "";
	mindex_--;
	
	for (int i = 0; i < mindex_; i++) {
		if (data_[i].active_ == false && data_[i + 1].active_) {
			data_[i].fname_ = data_[i + 1].fname_;
			data_[i].format_ = data_[i + 1].format_;
			data_[i].text_ = data_[i + 1].text_;
			data_[i].line_ = data_[i + 1].line_;
			data_[i].col_ = data_[i + 1].col_;
			data_[i].active_ = true;
			data_[i + 1].active_ = false;
		}
	}
		
	aindex_ = 0;
	if (mindex_ == 0) {
		mindex_ = 1;
		data_[0].active_ = true;
		data_[0].fname_ = "";
		data_[0].text_ = "";
		data_[0].line_ = 0;
		data_[0].col_ = 0;
		data_[0].format_ = FILE_UTF8;
	}
	return true;
}

QString BufMgr::getFName(int index) {
	if (index >= mindex_)
		return 0;
		
	return data_[index].fname_;
}

void BufMgr::setFName(QString fname, int index) {
	if (index >= mindex_)
		return;
		
	data_[index].fname_ = fname;
}


FILE_FORMAT BufMgr::getFormat(int index) {
	if (index > mindex_)
		return FILE_UTF8;
		
	return data_[index].format_;
}

void BufMgr::setActiveFName(QString fname) {
	data_[aindex_].fname_ = fname; // deep copy
}

QString BufMgr::getActiveFName() {
	return data_[aindex_].fname_;
}

void BufMgr::setActiveText(QString text) {
	data_[aindex_].text_ = text;
}

QString& BufMgr::getActiveText() {
	return data_[aindex_].text_;
}

int BufMgr::getActiveLine() {
	return data_[aindex_].line_;
}

void BufMgr::setActiveLine(int line) {
	data_[aindex_].line_ = line;
}


int BufMgr::getActiveCol() {
	return data_[aindex_].col_;
}

void BufMgr::setActiveCol(int col) {
	data_[aindex_].col_ = col;
}

void BufMgr::setActiveNext() {
	LOG("BufMgr old active index %d", aindex_);
	aindex_++;
	if (aindex_ >= mindex_)
		aindex_ = 0;
	LOG("BufMgr active index %d", aindex_);
}

bool BufMgr::anyBufChanged() {
	bool retval = false;
	for (int i = 0; i < mindex_; i++) {
		if (data_[i].fname_.startsWith("*")) {
			retval = true;
			break;
		}
	}
	return retval;
}

#define MAXS 500

QString BufMgr::roDebug() {
	QString retval = "Buffer Manager:\n";
	char buf[MAXS + 1];
	snprintf(buf, MAXS, "max buf supported %d, active index %d, max index %d\n",
		 MAXBUFMGR, aindex_, mindex_);
	retval += buf;

	for (int i = 0; i < mindex_; i++) {
		snprintf(buf, MAXS, "%d: %s, %s, %d/%d\n",
			 i,
			 (const char *) data_[i].fname_,
			 (data_[i].active_)? "active": "inactive",
			 data_[i].line_,
			 data_[i].col_);
		retval += buf;
	}

	return retval;
}
