//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************

#include "qapplication.h"
#include "ctags.h"
#include <time.h>
#include "debug.h"
#include "utils.h"
#include "global.h"

CtagsThread::CtagsThread(QObject *parent): 
#ifdef USING_QT4
	QThread(parent),
#endif
#ifdef USING_QT3
	QThread(),
#endif
	doit_(0), done_(0), parent_(parent) {
	

	msgout_ = "";
	outbuffer_ = "";
	outfile_ = "";
	active_ = false;

	LOG("ctags thread created");
}

CtagsThread::~CtagsThread() {
}

void CtagsThread::sendAlldone() {
#ifdef USING_QT4
	emit alldone();
#else
	QApplication::postEvent(parent_, new QCustomEvent(65432));
#endif
}

void CtagsThread::run() {
	LOG("ctags thread run");
	srand(time(NULL));

	loadTagFile();


	bool nodelta = true;
	while(1) {
		// reload tags command from qwerty;
		// loadTagFile will set the active_ flag
		if (msgin_ == "reload tags") {
			while (!ltag_.empty())
				ltag_.erase(ltag_.begin());
			
			loadTagFile();
			msgin_ = "";
			done_ = doit_;
		}
		
		// process ctag input if active_, there is no delta,
		// and there is something to do
		else if (active_ && nodelta && doit_ > done_) {
			LOG("ctags delta %d", doit_ - done_);
			// work on copyes of input data!
			// real input data can change due to the user pressing keys
			// during our ctags processing below!
			int inprogress = doit_;
			QString msgin_todo = msgin_;

			// try to find the keyword located in msgin
			list<Tag>::iterator it;
			list<Tag>::iterator start;
			list<Tag>::iterator end;
			msgout_ = "";
			int cnt = 0;
			Tag unique;
			LOG("ctag check word %s", (const char *) msgin_);
	
			if (msgin_todo < (*lmiddle1_).keyword_) {
				LOG("start from begin");
				start = ltag_.begin();
				end = lmiddle1_;
			}
			else if (msgin_todo < (*lmiddle2_).keyword_) {
				LOG("start from middle1");
				start = lmiddle1_;
				end = lmiddle2_;
			}
			else if (msgin_todo < (*lmiddle3_).keyword_) {
				LOG("start from middle2");
				start = lmiddle2_;
				end = lmiddle3_;
			}
			else if (msgin_todo < (*lmiddle4_).keyword_) {
				LOG("start from middle3");
				start = lmiddle3_;
				end = lmiddle4_;
			}
			else if (msgin_todo < (*lmiddle5_).keyword_) {
				LOG("start from middle4");
				start = lmiddle4_;
				end = lmiddle5_;
			}
			else if (msgin_todo < (*lmiddle6_).keyword_) {
				LOG("start from middle5");
				start = lmiddle5_;
				end = lmiddle6_;
			}
			else if (msgin_todo < (*lmiddle7_).keyword_) {
				LOG("start from middle6");
				start = lmiddle6_;
				end = lmiddle7_;
			}
			else {
				LOG("strat from middle7");
				start = lmiddle7_;
				end = ltag_.end();
			}

			for(it = start; it != end; ++it) {
				Tag t = *it;
				if (t.keyword_ == msgin_todo) {
					unique = t;
//					t.log();
					cnt++;
					// found entry, copy it in the result list
					msgout_ += t.file_ + "::" + t.line_number_ +
						   " " + t.line_text_ + "\n";
				}
				if (t.keyword_ > msgin_todo)
					break;
			}
			if (cnt == 1) {
//				unique.log();
				msgout_ = unique.file_ + "::" + unique.line_number_ + ":" + " single ctag definition, see below...\n";
				
				if (last_file1_ == unique.file_) {
					msgout_ += last_text1_;
				}
				else if (last_file2_ == unique.file_) {
					msgout_ += last_text2_;
				}
				else if (last_file3_ == unique.file_) {
					msgout_ += last_text3_;
				}
				else if (last_file4_ == unique.file_) {
					msgout_ += last_text4_;
				}
				else {
					LOG("ctag load  file %s", (const char *) unique.file_);
					char *text = UTILS::read_file_malloc((const char *) unique.file_);
					msgout_ += text;
					int r = rand() % 4;
					if (r == 0) {
						last_file1_ = unique.file_;
						last_text1_ = text;
					}
					else if (r == 1) {
						last_file2_ = unique.file_;
						last_text2_ = text;
					}
					else if (r == 2) {
						last_file3_ = unique.file_;
						last_text3_ = text;
					}
					else {
						last_file4_ = unique.file_;
						last_text4_ = text;
					}
					
					delete[] text;
				}
			}
			
			sendAlldone();
			sleep(1); // some delay for qwerty to consume the information
			done_ = inprogress;
		}
		
		// process nodelta:
		//  - set nodelta if during the last one second sleep here
		//    was no key pressed in qwerty
		//  - reset nodelta otherwise
		// this functionality is intorduced to make sure we do not
		// try to process ctags while the user is moving the cursor
		else {
			
			int tmp = doit_;
			sleep(2);
			if (doit_ == tmp)
				nodelta = true;
			else
				nodelta = false;
		}
	}
}

void CtagsThread::startProcessing(QString ctagword) {
//	if (!active_)
//		return;

	if (msgin_ != ctagword) {
		msgin_ = ctagword;
		LOG("start processing %s", (const char *) ctagword);
		doit_++;
	}
}

#define MAXS 1000

bool CtagsThread::readOneLine(FILE *fp, char *buf, int size) {
	int i = 0;
	bool retval = false;
	while (1) {
		*buf = getc(fp);
		if (i >= size || *buf == EOF) {
			retval = false;
			break;
		}
		if (*buf == '\n' || *buf == '\r') {
			retval = true;
			break;
		}
		buf++;
		i++;
	}
	*buf = '\0';
	return retval;
}


void CtagsThread::loadTagFile(FILE *fp) {
	char buf[MAXS + 1];
	
	int state = 0;
	char filename[MAXS + 1];
	QString qfilename;
	char text[MAXS + 1];
	char keyword[MAXS + 1];
	char line[MAXS + 1];
	while (readOneLine(fp, buf, MAXS)) {
		if (*buf == 0x0c) {
			state = 1;
			continue;
		}
		
		switch (state) {
			case 0: {
				snprintf(filename, MAXS, "Error 1: bad input tag file\n%s\n", buf);
				msgout_ = filename;
				sendAlldone();
				active_ = false;
				return;
			}
			case 1: {// we should have a file name here
				char *ptr = buf;
				while (*ptr != ',' && *ptr != '\0') {
//printf("%x, ", *ptr);
					ptr++;
				}
//printf("%x, ", *ptr);
				if (*ptr == '\0') {
					snprintf(filename, MAXS, "Error 2: bad input tag file\n%s\n", buf);
					msgout_ = filename;
					sendAlldone();
					active_ = false;
					return;
				}
				*ptr = '\0';
				strcpy(filename, buf);
				qfilename = QString(Global::unixHomeConvert(QString(filename)));
//				LOG("file %s ### %s", filename, (const char *) qfilename); 
				state = 2;
			}
			break;
			case 2: {// we should have a tag here
				char *ptr = buf;
				while (*ptr != 0x7f && *ptr != '\0') {
//printf("%x, ", *ptr);
					ptr++;
				}
//printf("%x, ", *ptr);
				if (*ptr == '\0') {
					snprintf(filename, MAXS, "Error 3: bad input tag file\n%s\n", buf);
					msgout_ = filename;
					sendAlldone();
					active_ = false;
					return;
				}
				*ptr = '\0';
				ptr++;
				strcpy(text, buf);

				char *ptr1 = ptr;
				while (*ptr != 0x01 && *ptr != '\0')
					ptr++;
				if (*ptr == '\0') {
					snprintf(filename, MAXS, "Error 4: bad input tag file\n%s\n", buf);
					msgout_ = filename;
					sendAlldone();
					active_ = false;
					return;
				}
				*ptr = '\0';
				ptr++;
				strcpy(keyword, ptr1);

				ptr1 = ptr;
				while (*ptr != ',' && *ptr != '\0')
					ptr++;
				if (*ptr == '\0') {
					snprintf(filename, MAXS, "Error 5: bad input tag file\n%s\n", buf);
					msgout_ = filename;
					sendAlldone();
					active_ = false;
					return;
				}
				*ptr = '\0';
				ptr++;
				strcpy(line, ptr1);
				ltag_.push_back(Tag(qfilename, QString(keyword), QString(text), QString(line)));
//				printf("file: %s\nkeyword: %s\ntext: %s\nline: %s\n",
//					filename, keyword, text, line);
			}
			break;
			default:
				break;
		}
	}
}



void CtagsThread::loadTagFile() {
	FILE *fp;
	
	// open local tag file
	fp = fopen("TAGS", "r");
	active_ = false;
	if (fp == 0) {
		msgout_ = "user tag file not found or empty";
		sendAlldone();
	}
	else {
		loadTagFile(fp);
		fclose(fp);
		msgout_ = "tag file readed";
		sendAlldone();
	}		
	
	// open glibc tag file
	if (Global::instance()->load_glibc_tags) {
		char *libc_fname = Global::getLibcFname();
		fp = fopen(libc_fname, "r");
		delete[] libc_fname;
		
		if (fp == 0) {
			LOG("libc tag file not found or empty");
			msgout_ = "libc tag file not found or empty";
			sendAlldone();
		}
		else {
			loadTagFile(fp);	
			fclose(fp);
			msgout_ = "tag file readed";
			sendAlldone();
		}
	}
	
	if (ltag_.empty()) {
		msgout_ = "ctags inactive";
		sendAlldone();
		active_ = false;
		return;
	}
	
	// sort the list
	ltag_.sort();
	msgout_ = "tag file sorted";
	sendAlldone();
	
	unsigned len = ltag_.size() / 8;
	lmiddle1_ = ltag_.begin();
	for (unsigned i = 0; i < len; i++, ++lmiddle1_);
	lmiddle2_ = lmiddle1_;
	for (unsigned long i = 0; i < len; i++, ++lmiddle2_);
	lmiddle3_ = lmiddle2_;
	for (unsigned long i = 0; i < len; i++, ++lmiddle3_);
	lmiddle4_ = lmiddle3_;
	for (unsigned long i = 0; i < len; i++, ++lmiddle4_);
	lmiddle5_ = lmiddle4_;
	for (unsigned long i = 0; i < len; i++, ++lmiddle5_);
	lmiddle6_ = lmiddle5_;
	for (unsigned long i = 0; i < len; i++, ++lmiddle6_);
	lmiddle7_ = lmiddle6_;
	for (unsigned long i = 0; i < len; i++, ++lmiddle7_);

	
//	list<Tag>::iterator it;
//	for (it = ltag_.begin(); it != lmiddle1_; ++it)
//		LOG("middle1:%s", (const char *) (*it).keyword_);
//	for (it = lmiddle1_; it != lmiddle2_; ++it)
//		LOG("middle2:%s", (const char *) (*it).keyword_);
//	for (it = lmiddle2_; it != lmiddle3_; ++it)
//		LOG("middle3:%s", (const char *) (*it).keyword_);
//	for (it = lmiddle3_; it != ltag_.end(); ++it)
//		LOG("middle4:%s", (const char *) (*it).keyword_);
	
	msgout_ = "ctags active";
	sendAlldone();
	active_ = true;
}

QString CtagsThread::roDebug() {
	QString retval = "ctags: ";
	retval += (active_)? "active": "inactive";
	retval += QString(", doit/done %1/").arg(doit_);
	retval += QString("%1\n").arg(done_);
	retval += "1: " + last_file1_ + "\n";
	retval += "2: " + last_file2_ + "\n";
	retval += "3: " + last_file3_ + "\n";
	retval += "4: " + last_file4_ + "\n";
	return retval;
}


//#endif // USING_QT4

