//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************

#ifndef CTAGS_H
#define CTAGS_H
#include <qwidget.h>
#include <qthread.h>
#include <list>
using namespace std;
#include "debug.h"

//porting to qt3 info:
//http://doc.trolltech.com/3.3/qthread.html
//http://doc.trolltech.com/3.3/qcustomevent.html
//example using timers: http://doc.trolltech.com/3.3/timers.html
//thread example: http://doc.trolltech.com/solutions/3/qtreadwritemutex/qtreadwritemutex-example-fileaccess.html



class CtagsThread: public QThread {
#ifdef USING_QT4
	Q_OBJECT
#endif
private:
	struct Tag {
		QString file_;
		QString keyword_;
		QString line_text_;
		QString line_number_;

		Tag() {}

//		Tag(const QString &f, const QString &k, const QString &lt, const QString &ln):
//			file_(f), keyword_(k), line_text_(lt), line_number_(ln) {}
		Tag(QString f, QString k, QString lt, QString ln):
			file_(f), keyword_(k), line_text_(lt), line_number_(ln) {}

		void log() {
			LOG("***tag***");
			LOG("tag.file: %s", (const char *) file_);
			LOG("tag.keyword: %s", (const char *) keyword_);
			LOG("tag.line text: %s", (const char *) line_text_);
			LOG("tag.line num: %s", (const char *) line_number_);
			LOG("*********");
		}
		
		bool operator < (Tag &tag) {
			return keyword_ < tag.keyword_;
		}
	};
public:
	CtagsThread(QObject *parent);
	~CtagsThread();
	void startProcessing(QString ctagword);
	QString getMsg() {
		return msgout_;
	}
	bool isActive() {
		return active_;
	}
	int getTagsNumber() {
		return ltag_.size();
	}
	QString roDebug();

	void sendAlldone();
#ifdef USING_QT4
signals:
	void alldone();
#endif
protected:
	void run();
	bool readOneLine(FILE *fp, char *buf, int size);
	void loadTagFile();
	void loadTagFile(FILE *fp);

	volatile int doit_;
	volatile int done_;

private:
	// parent  where to send the event (in qt3)
	QObject *parent_;
	// tag list
	list<Tag> ltag_;
	list<Tag>::iterator lmiddle1_;
	list<Tag>::iterator lmiddle2_;
	list<Tag>::iterator lmiddle3_;
	list<Tag>::iterator lmiddle4_;
	list<Tag>::iterator lmiddle5_;
	list<Tag>::iterator lmiddle6_;
	list<Tag>::iterator lmiddle7_;
	// messages
	QString msgin_;
	QString msgout_;
	QString outbuffer_;
	QString outfile_;
	bool active_;
	// last file buffers
	QString last_file1_;
	QString last_text1_;
	QString last_file2_;
	QString last_text2_;
	QString last_file3_;
	QString last_text3_;
	QString last_file4_;
	QString last_text4_;
};

#endif
