//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************

#include "dialogs.h"
#include <qmessagebox.h>
#include <qgroupbox.h>
#include "debug.h"

#ifdef USING_QT4
#include <Q3VBoxLayout>
#include <q3hbox.h>
#include <q3accel.h>
#include <QLabel>
#endif
#ifdef USING_QT3
#include <qlayout.h>
#include <qhbox.h>
#include <qaccel.h>
#include <qlabel.h>
#endif

#include <qlineedit.h>
#include <qcheckbox.h>
#include <qpushbutton.h>


//*****************************************************************************/
// line dialog
//*****************************************************************************/
LineDialog::LineDialog(): QDialog(0, "Enter line number", true) {
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	Q3HBox *hbox = new Q3HBox(this);
	hbox->setSpacing(10);
	label_ = new QLabel("Line: ", hbox);
	text_ = new QLineEdit(hbox);

	// accept/reject
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	button_ok_ = new QPushButton("Ok", hbox2);
	button_cancel_ = new QPushButton("Cancel", hbox2);

	connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
}

int LineDialog::getLine() {
	int retval;

	if (text_->text().isEmpty() == true)
		retval = -1;
	else {
		QString str = text_->text();
		bool ok;
		retval = str.toInt(&ok);
		if (!ok || retval <= 0) {
			retval = -1;
			QMessageBox::warning(0,"Error", "Not a valid line number");
		}
	}
	return retval;
}

//*****************************************************************************/
// bookmark dialog
//*****************************************************************************/
BookmarkDialog::BookmarkDialog(): QDialog(0, "Enter bookmark number", true) {
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	Q3HBox *hbox = new Q3HBox(this);
	hbox->setSpacing(10);
	label_ = new QLabel("Bookmark: ", hbox);
	text_ = new QLineEdit(hbox);

	// accept/reject
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	button_ok_ = new QPushButton("Ok", hbox2);
	button_cancel_ = new QPushButton("Cancel", hbox2);

	connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
}

int BookmarkDialog::getBookmark() {
	int retval;

	if (text_->text().isEmpty() == true)
		retval = -1;
	else {
		QString str = text_->text();
		bool ok;
		retval = str.toInt(&ok);
		if (!ok || retval < 0 || retval >= MAXBOOKMARKS) {
			retval = -1;
			QMessageBox::warning(0,"Error", "Not a valid bookmark number");
		}
	}
	return retval;
}


//*****************************************************************************/
// www dialog
//*****************************************************************************/
WwwDialog::WwwDialog(): QDialog(0, "www location", true) {
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	label_ = new QLabel("Please enter the URL of a file you want to download", hbox1);

	Q3HBox *hbox = new Q3HBox(this);
	hbox->setSpacing(10);
	text_ = new QLineEdit(hbox);
	button_ = new QPushButton("Ok", hbox);

	connect(button_, SIGNAL(clicked()), SLOT(accept()));
}

QString WwwDialog::getWord() {
	QString retval;

	if (text_->text().isEmpty() == true)
		retval = "";
	else {
		retval = text_->text();
	}
	return retval;
}

//*****************************************************************************/
// status dialog
//*****************************************************************************/
StatusDialog::StatusDialog(QFont &fnt, char *msg): QDialog(0, "File status", true) {
	ASSERT(msg != 0);
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	Q3HBox *hbox = new Q3HBox(this);

	QString qstr = QString::fromUtf8(msg);

	label_ = new QLabel(qstr, hbox);
	label_->setFont(fnt);
	button_ = new QPushButton("Ok", hbox);

	connect(button_, SIGNAL(clicked()), SLOT(accept()));
}



//*****************************************************************************/
// basic unicode dialog
//*****************************************************************************/
BasicUnicodeDialog::BasicUnicodeDialog(QString title): QDialog(0, title, true) {
	text_ = 0;

	Q3Accel *a = new Q3Accel(this);		  // create accels for myWindow
	a->connectItem(a->insertItem(Qt::ALT+Qt::Key_A), this, SLOT(unicode_a()));
	a->connectItem(a->insertItem(Qt::SHIFT+Qt::ALT+Qt::Key_A), this, SLOT(unicode_A()));
	a->connectItem(a->insertItem(Qt::ALT+Qt::Key_S), this, SLOT(unicode_s()));
	a->connectItem(a->insertItem(Qt::SHIFT+Qt::ALT+Qt::Key_S), this, SLOT(unicode_S()));
	a->connectItem(a->insertItem(Qt::ALT+Qt::Key_Q), this, SLOT(unicode_q()));
	a->connectItem(a->insertItem(Qt::SHIFT+Qt::ALT+Qt::Key_Q), this, SLOT(unicode_Q()));
	a->connectItem(a->insertItem(Qt::ALT+Qt::Key_T), this, SLOT(unicode_t()));
	a->connectItem(a->insertItem(Qt::SHIFT+Qt::ALT+Qt::Key_T), this, SLOT(unicode_T()));
	a->connectItem(a->insertItem(Qt::ALT+Qt::Key_I), this, SLOT(unicode_i()));
	a->connectItem(a->insertItem(Qt::SHIFT+Qt::ALT+Qt::Key_I), this, SLOT(unicode_I()));
}

QString BasicUnicodeDialog::getWord() {
	if (text_ == 0)
		return "";

	QString retval;

	if (text_->text().isEmpty() == true)
		retval = "";
	else {
		retval = text_->text();
	}
	return retval;
}


void BasicUnicodeDialog::unicode_a() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	QString str = QString::fromUtf8("ă");
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_A() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	QString str = QString::fromUtf8("Ă");
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_s() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	QString str = QString::fromUtf8("ş");
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_S() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	QString str = QString::fromUtf8("Ş");
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_q() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	QString str = QString::fromUtf8("â");
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_Q() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	QString str = QString::fromUtf8("Â");
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_t() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	QString str = QString::fromUtf8("ţ");
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_T() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	QString str = QString::fromUtf8("Ţ");
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_i() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	QString str = QString::fromUtf8("î");
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_I() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	QString str = QString::fromUtf8("Î");
	text_->insert(str);
	text_->cursorForward(false);
}






//*****************************************************************************/
// search dialog
//*****************************************************************************/
SearchDialog::SearchDialog(): BasicUnicodeDialog("Enter search word") {
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	// target word
	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	label_ = new QLabel("Target: ", hbox1);
	text_ = new QLineEdit(hbox1);

	Q3HBox *hbox3 = new Q3HBox(this);
	ignore_case_ = new QCheckBox(hbox3);
	label1_ = new QLabel("Ignore case", hbox3);

	// accept/reject
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	button_ok_ = new QPushButton("Ok", hbox2);
	button_cancel_ = new QPushButton("Cancel", hbox2);


	connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
}

bool SearchDialog::getIgnoreCase() {
	return ignore_case_->isChecked();
}





//*****************************************************************************/
// search dialog
//*****************************************************************************/
ReplaceDialog::ReplaceDialog(QString tofind, QString replacement):
	BasicUnicodeDialog("Replace") {
	
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	// target word
	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	label1_ = new QLabel("Target: ", hbox1);
	text1_ = new QLineEdit(hbox1);
	text1_->setText(tofind);

	Q3HBox *hbox2 = new Q3HBox(this);
	hbox1->setSpacing(10);
	label2_ = new QLabel("Replace with: ", hbox2);
	text2_ = new QLineEdit(hbox2);
	text2_->setText(replacement);

	// accept/reject
	Q3HBox *hbox3 = new Q3HBox(this);
	hbox3->setSpacing(10);
	button_findnext_ = new QPushButton("Find Next", hbox3);
	button_replace_ = new QPushButton("Replace", hbox3);
	button_replaceall_ = new QPushButton("Replace All", hbox3);
	button_cancel_ = new QPushButton("Cancel", hbox3);


	connect(button_findnext_, SIGNAL(clicked()), this, SLOT(clicked_findnext()));
	connect(button_replace_, SIGNAL(clicked()), this, SLOT(clicked_replace()));
	connect(button_replaceall_, SIGNAL(clicked()), this, SLOT(clicked_replaceall()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(clicked_cancel()));
//	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));

	text_ = 0;
	action_ = ACTION_NONE;
}

bool ReplaceDialog::hasFocus() {
	if (text1_->hasFocus())
		text_ = text1_;
	else if (text2_->hasFocus())
		text_ = text2_;
	else
		text_ = 0;

	return true;	
}

void ReplaceDialog::printAction() {
	if (action_ == ACTION_FINDNEXT)
		LOG("ReplaceDialog::ACTION_FINDNEXT");
	else if (action_ == ACTION_REPLACE)
		LOG("ReplaceDialog::ACTION_REPLACE");
	else if (action_ == ACTION_REPLACEALL)
		LOG("ReplaceDialog::ACTION_REPLACEALL");
	else if (action_ == ACTION_CANCEL)
		LOG("ReplaceDialog::ACTION_CANCEL");
	else
		LOG("ReplaceDialog::ACTION_NONE");
	
}
void ReplaceDialog::clicked_findnext() {
	action_ = ACTION_FINDNEXT;
}

void ReplaceDialog::clicked_replace() {
	action_ = ACTION_REPLACE;
}

void ReplaceDialog::clicked_replaceall() {
	action_ = ACTION_REPLACEALL;
}

void ReplaceDialog::clicked_cancel() {
	action_ = ACTION_CANCEL;
}

QString ReplaceDialog::getWordToFind() {
	if (text1_ == 0)
		return "";

	QString retval;

	if (text1_->text().isEmpty() == true)
		retval = "";
	else {
		retval = text1_->text();
	}
	return retval;
}

QString ReplaceDialog::getReplacement() {
	if (text2_ == 0)
		return "";

	QString retval;

	if (text2_->text().isEmpty() == true)
		retval = "";
	else {
		retval = text2_->text();
	}
	return retval;
}
