//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************

#include "diffdialog.h"
#include <qmessagebox.h>
#include <qgroupbox.h>

#ifdef USING_QT3
#include <qfiledialog.h>
#endif
#ifdef USING_QT4
#include <q3filedialog.h>
#endif


#ifdef USING_QT3
#include <qlayout.h>
#include <qlabel.h>
#include <qhbox.h>
#include <qaccel.h>
#include <qmultilineedit.h>
#endif
#ifdef USING_QT4
#include <Q3VBoxLayout>
#include <QLabel>
#include <q3hbox.h>
#include <q3accel.h>
#include <q3multilineedit.h>
#endif

#include <qlineedit.h>
#include <qcheckbox.h>
#include <qpushbutton.h>

#include "global.h"


//*****************************************************************************/
// diff dialog
//*****************************************************************************/
DiffDialog::DiffDialog(): QDialog(0, "diff", true) {
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);
	
	// file1
	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	label1_ = new QLabel("File 1: ", hbox1);
	file1_ = new QLineEdit(hbox1);
	button_browse1_ = new QPushButton("browse", hbox1);

	// file2
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	label2_ = new QLabel("File 2: ", hbox2);
	file2_ = new QLineEdit(hbox2);
	button_browse2_ = new QPushButton("browse", hbox2);

	// accept/reject
	Q3HBox *hbox3 = new Q3HBox(this);
	hbox3->setSpacing(10);
	button_ok_ = new QPushButton("Ok", hbox3);
	button_cancel_ = new QPushButton("Cancel", hbox3);

	connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
	connect(button_browse1_, SIGNAL(clicked()), this, SLOT(clicked_browse1()));
	connect(button_browse2_, SIGNAL(clicked()), this, SLOT(clicked_browse2()));
}

QString DiffDialog::getFile1() {
	if (file1_ == 0)
		return "";

	QString retval;

	if (file1_->text().isEmpty() == true)
		retval = "";
	else {
		retval = file1_->text();
	}
	return retval;
}

QString DiffDialog::getFile2() {
	if (file2_ == 0)
		return "";

	QString retval;

	if (file2_->text().isEmpty() == true)
		retval = "";
	else {
		retval = file2_->text();
	}
	return retval;
}

void DiffDialog::clicked_browse1() {
	QString file = Q3FileDialog::getOpenFileName( QString::null, QString::null, this );
	file1_->setText(file);
}

void DiffDialog::clicked_browse2() {
	QString file = Q3FileDialog::getOpenFileName( QString::null, QString::null, this );
	file2_->setText(file);
}



DiffWorksDialog::DiffWorksDialog(QFont &fnt, QString file1, QString file2): QDialog(0, "diffworks", true) {
LOG("file1 %s, file2 %s", (const char *) file1, (const char *) file2);
//	setMinimumHeight(400);
//	setMinimumWidth(600);
	Global *g = Global::instance();

	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);
	
	// file1
	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	f1_ = new QLabel("-: " + file1, hbox1);
	f1_->setPaletteForegroundColor(g->c_tag2);

	// file2
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	f2_ = new QLabel("+: " + file2, hbox2);
	f2_->setPaletteForegroundColor(g->c_tag1);

	// diff
	Q3HBox *hbox3 = new Q3HBox(this);
	hbox3->setSpacing(10);
	text_ = new Q3MultiLineEdit(hbox3);
	syntax_ = getSyntax(".diff", text_);

	// accept/reject
	Q3HBox *hbox4 = new Q3HBox(this);
	hbox4->setSpacing(10);
	button_next_ = new QPushButton("Next", hbox4);
	button_previous_ = new QPushButton("Previous", hbox4);
	button_ok_ = new QPushButton("Ok", hbox4);
	button_cancel_ = new QPushButton("Cancel", hbox4);

	connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
	connect(button_next_, SIGNAL(clicked()), this, SLOT(clicked_next()));
	connect(button_previous_, SIGNAL(clicked()), this, SLOT(clicked_previous()));

	text_->setFont(fnt);
	text_->setWordWrap(Q3TextEdit::NoWrap);
	text_->setReadOnly(TRUE);

	// colors
	text_->setPaletteBackgroundColor(g->c_background);
	text_->setPaletteForegroundColor(g->c_foreground);
	QPalette qp = text_->palette();
	qp.setColor(QPalette::Active, QColorGroup::Highlight, g->c_highlight);
	qp.setColor(QPalette::Inactive, QColorGroup::Highlight, g->c_highlight);
	text_->setPalette(qp);

	text_->setText(QString::fromUtf8(doDiff((const char *) file1, (const char *) file2)));
	
	resize(Global::instance()->diff_x, Global::instance()->diff_y);
	
	// set initial selection
	int line;
	int col;
	text_->getCursorPosition(&line, &col);
	QString str = text_->textLine(line);
	if (str.at(0) == ' ')
		clicked_next();
	else {
		int start = line;
		int end = start + 1;
		while (!text_->textLine(end).isNull() &&
		       !text_->textLine(end).isEmpty() &&
		       text_->textLine(end).at(0) != ' ')
			end++;
		
		text_->setSelection(start, 0, end, 0);
	}	
}

void DiffWorksDialog::clicked_next() {
	int line;
	int col;
	text_->getCursorPosition(&line, &col);
	QString str = "";
	bool found = false;
	while (1) {
		line++;
		text_->setCursorPosition(line, 0, false);
		if (text_->atEnd())
			return;		
		str = text_->textLine(line);
		if (str.at(0) != ' ') {
			found = true;
			break;
		}
	}
	
	int start = line;
	int end = line;
	while (found) {
		line++;
		end = line;
		text_->setCursorPosition(line, 0, false);
		if (text_->atEnd())
			return;		
		str = text_->textLine(line);
		if (str.isNull() || str.isEmpty())
			break;
		if (str.at(0) == ' ')
			break;
	}
	
	text_->setSelection(start, 0, end, 0);
	update();
}

void DiffWorksDialog::clicked_previous() {
	int line;
	int col;
	text_->getCursorPosition(&line, &col);
	line--;
	QString str = text_->textLine(line);
	while (line >= 0 && str.at(0) != ' ') {
		line--;
		str = text_->textLine(line);
	}
		
	bool found = false;
	while (line >= 0) {
		line--;
		text_->setCursorPosition(line, 0, false);
		str = text_->textLine(line);
		if (str.at(0) != ' ') {
			found = true;
			break;
		}
	}
	
	int start = line;
	int end = line;
	while (found && line >= 0) {
		line--;
		end = line;
		text_->setCursorPosition(line, 0, false);
		if (text_->atEnd())
			return;		
		str = text_->textLine(line);
		if (str.isNull() || str.isEmpty())
			break;
		if (str.at(0) == ' ')
			break;
	}
	text_->setSelection(end + 1, 0, start + 1, 0);
	update();
}

void DiffWorksDialog::resizeEvent( QResizeEvent * )
{
	LOG("DiffWorksDialog::resizeEvent sizes %d %d", width(), height());
//	if (e_ && m_ )
//		e_->setGeometry( 0, m_->height(), width(), height() - m_->height());
	Global::instance()->diff_x = width();
	Global::instance()->diff_y = height();
}
