//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************

#ifndef HASHDB_H
#define HASHDB_H

#include "global.h"
//#include "utils.h"
#include "utf8.h"
#ifdef _WIN32
#define DF1 "c:\\rospell\\dictionary.ro"
#define DF2 "c:\\rospell\\dictionary.fixed"
#define DF3 "c:\\rospell\\dictionary.en"

#else
#include "../../dictfiles.h"
#endif

template <unsigned ENTRIES, unsigned HASH_MASK>
class HashDb {
	class HString {
	public:
		HString(char *str) {
			ASSERT(str != 0);
			int len = std::strlen(str) + 1;
			str_ = new char[len + 1];
			strcpy(str_, str);
		}
		~HString() {
			delete[] str_;
		}
		// debug print
		void print() {
			printf("%s\n", str_);
		}
		void strcut(char *str) {
			ASSERT(str != 0);
			ASSERT(str_ != 0);
			unsigned len = std::strlen(str) + std::strlen(str_) + 1 + 1; // 1 for ' ' and 1 for '\0'
			char *newstr = new char[len];
			std::strcpy(newstr, str_);
			std::strcat(newstr, " ");
			std::strcat(newstr, str);
			ASSERT(len == (std::strlen(newstr) + 1));
			delete[] str_;
			str_ = newstr;
		}
		bool strstr(char *str) {
			ASSERT(str != 0);
			return private_strstr(str_, str);
		}
		unsigned strlen() {
			return std::strlen(str_);
		}
		char *strget() {
			return str_;
		}

	private:
		bool private_strstr(char *input, char *search) {
			ASSERT(input != 0);
			ASSERT(search != 0);

			char *ptr;
			if ((ptr = std::strstr(input, search)) != 0) {
				if (((unsigned) ptr == (unsigned) input || *(ptr - 1) == ' ') &&
				    (*(ptr + std::strlen(search)) == '\0' || *(ptr + std::strlen(search)) == ' '))
					return true;
				else if (*(ptr + 1) != '\0')
					return private_strstr(ptr + 1, search);
				else
					return false;
			} else
				return false;
		}

		char *str_;
	};

public:
	HashDb() {
		for (unsigned i = 0; i < ENTRIES; i++)
			array_[i] = 0;
	}
	~HashDb() {
		for (unsigned i = 0; i < ENTRIES; i++)
			if (array_[i] != 0)
				delete array_[i];
	}

	// debug print
	void print() {
		for (unsigned i = 0; i < ENTRIES; i++)
			if (array_[i] != 0)
				array_[i]->print();
	}

	void setString(char *str, bool fixed = false) {
		PROFILE("HashDB::setString");
		ASSERT(str != 0);
		unsigned entry = hash(str);
		ASSERT(entry < ENTRIES);

		// convert string to lower
		char *lstr = UTF8::strtolower(str);
		char *ptr = (fixed)? str: lstr;
		if (array_[entry] == 0)
			array_[entry] = new HString(ptr);
		else
			array_[entry]->strcut(ptr);
		delete[] lstr;
	}

	// str - utf8 string to look for
	bool findString(char *str, bool fixed = false) {
		PROFILE("HashDB::findString");
		ASSERT(str != 0);
		unsigned entry = hash(str);
		ASSERT(entry < ENTRIES);
		bool retval = false;

		// convert string to lower
		char *lstr = UTF8::strtolower(str);
		char *ptr = (fixed)? str: lstr;
		if (array_[entry] == 0)
			retval = false;
		else
			retval = array_[entry]->strstr(ptr);
		delete[] lstr;
		return retval;
	}

	// str - utf8 string to look for
	void findLookalike(char *str, char *str1, char *str2, char *str3, char *str4, char *str5) {
		PROFILE("HashDB::findString");
		ASSERT(str != 0);
		unsigned entry = hash(str);
		ASSERT(entry < ENTRIES);
		*str1 = '\0';
		*str2 = '\0';
		*str3 = '\0';
		*str4 = '\0';
		*str5 = '\0';

		// convert string to lower
		char *lstr = UTF8::strtolower(str);
		if (array_[entry] != 0) {
			char *line = array_[entry]->strget();
			// first, translate the strings to ascii
			char *asciiline = UTF8::strtoascii(line);
			ASSERT(asciiline != 0);
			char *asciiword = UTF8::strtoascii(lstr);
			ASSERT(asciiword != 0);
			// look for translated strings

			char *ptr1 = 0;
			char *ptr2 = 0;
			char *ptr3 = 0;
			char *ptr4 = 0;
			char *ptr5 = 0;

			ptr1 = strstr(asciiline, asciiword);
			if (ptr1 != 0) {
				ptr2 = strstr(ptr1 + 1, asciiword);
				if (ptr2 != 0) {
					ptr3 = strstr(ptr2 + 1, asciiword);
					if (ptr3 != 0) {
						ptr4 = strstr(ptr3 + 1, asciiword);
						if (ptr4 != 0) {
							ptr5 = strstr(ptr4 + 1, asciiword);
						}
					}
				}
			}

			copyFromAscii(asciiline, ptr1, line, str1);
			copyFromAscii(asciiline, ptr2, line, str2);
			copyFromAscii(asciiline, ptr3, line, str3);
			copyFromAscii(asciiline, ptr4, line, str4);
			copyFromAscii(asciiline, ptr5, line, str5);

			reorderNoEmpty(str1, str2, str3, str4, str5);
			reorderNoEmpty(str1, str2, str3, str4, str5);
			reorderNoEmpty(str1, str2, str3, str4, str5);
			reorderNoEmpty(str1, str2, str3, str4, str5);

			delete[] asciiline;
			delete[] asciiword;

			// convert upper first letter if necessary
			if (UTF8::isupper(str)) {
				UTF8::toupper(str1);
				UTF8::toupper(str2);
				UTF8::toupper(str3);
				UTF8::toupper(str4);
				UTF8::toupper(str5);
			}
		}
		delete[] lstr;
	}

	bool putFile(char *fname) {
		ASSERT(fname != 0);
		LOG("save dictionary file %s", fname);

		unsigned len = 0;
		for (unsigned i = 0; i < ENTRIES; i++)
			if (array_[i] == 0)
				len += 1;
			else
				len += array_[i]->strlen() + 1;

		char *str = new char[len + 1];
		char *ptr = str;
		for (unsigned i = 0; i < ENTRIES; i++) {
			if (array_[i] == 0) {
				*ptr = '\n';
				ptr++;
			} else {
				std::strcpy(ptr, array_[i]->strget());
				ptr += std::strlen(ptr);
				*ptr = '\n';
				ptr++;
			}
		}
		*ptr = '\0';
		bool retval = UTILS::write_file(fname, str);
		delete[] str;
		return retval;
	}

	bool getFile(char *fname) {
		ASSERT(fname != 0);
		LOG("load dictionary file %s", fname);
		char *str = UTILS::read_file_malloc(fname);
		if (str == 0)
			return false;
LOG("read %u chars from file", strlen(str));
		for (unsigned i = 0; i < ENTRIES; i++)
			if (array_[i] != 0)
				delete array_[i];

		char *ptr = str;
		for (unsigned i = 0; i < ENTRIES; i++) {
			if (*ptr == '\n')
				ptr++;
			else {
				char *ptmp = ptr;
				while (*ptmp != '\n')
					ptmp++;
				unsigned len = (unsigned) ptmp - (unsigned) ptr;
				char *newstr = new char[len + 1];
				strncpy(newstr, ptr, len);
				newstr[len] = '\0';
				array_[i] = new HString(newstr);
				ptr = ptmp + 1;
				delete[] newstr;
			}
		}
		delete[] str;
		return true;
	}


private:
	void reorderNoEmpty(char *str1, char *str2, char *str3, char *str4, char *str5) {
		if (*str1 == '\0' && *str2 != '\0') {
			strcpy(str1, str2);
			*str2 = '\0';
		}
		if (*str2 == '\0' && *str3 != '\0') {
			strcpy(str2, str3);
			*str3 = '\0';
		}
		if (*str3 == '\0' && *str4 != '\0') {
			strcpy(str3, str4);
			*str4 = '\0';
		}
		if (*str4 == '\0' && *str5 != '\0') {
			strcpy(str4, str5);
			*str5 = '\0';
		}
	}

	void copyFromAscii(char *asciiline, char *ptrascii, char *utfline, char *dest) {
		ASSERT(asciiline != 0);
		ASSERT(dest != 0);
		if (ptrascii != 0 &&
		    ((unsigned) (ptrascii - 1) == (unsigned) asciiline ||
			*(ptrascii - 1) == ' ')) {

			char *ptm = ptrascii;
			int cnt = 0;
			while ((unsigned) ptm > (unsigned) asciiline) {
				if (*ptm == ' ')
					cnt++;
				ptm--;
			}
//			LOG("space cnt = %d\n", cnt);
			int tmpcnt = 0;
			ptm = utfline;
			while (*ptm != '\0' && tmpcnt < cnt) {
				if (*ptm == ' ')
					tmpcnt++;
				ptm++;
			}
			ASSERT(*ptm != '\0');
//			LOG("space position = %s\n", ptm);
			while (*ptm != '\0' && *ptm != ' ') {
				*dest = *ptm;
				dest++;
				ptm++;
			}
			*dest = '\0';
		} else
			*dest = '\0';
	}

	unsigned hash(char *str) {
		ASSERT(str != 0);

		// first, translate the string to ascii
		char *astr1 = UTF8::strtoascii(str);
		ASSERT(astr1 != 0);
		// make string lower case
		char *astr = UTF8::strtolower(astr1);
		ASSERT(astr != 0);
		delete[] astr1;

		unsigned retval = phHash(astr);
		delete[] astr;
		return retval & HASH_MASK;
	}

	// Paul Hsieh's hash function - processing data on 16 bits
	// http://www.azillionmonkeys.com/qed/hash.html
	const unsigned short getshort(const char *d) {
		return *((const unsigned short *) (d));
	}
	unsigned int phHash (const char * data) {
		unsigned long hash;
		int i, rem;
		// len originally passed as a parameter: the number of chars in data
		int len = std::strlen(data);

		if (len <= 0 || data == NULL)	return 0;

		hash = (unsigned long) len; /* Avoid 0 -> 0 trap */
		rem = len & 3;
		len >>= 2;

		/* Main loop */
		for (i=0; i < len; i++) {
			hash += getshort (data);
			data += 2*sizeof (char);
			hash ^= hash << 16;
			hash ^= getshort (data) << 11;
			data += 2*sizeof (char);
			hash += hash >> 11;
		}

		/* Handle end cases */
		switch (rem) {
			case 3: hash += getshort (data);
				hash ^= hash << 16;
				hash ^= data[2 * sizeof (char)] << 18;
				hash += hash >> 11;
				break;
			case 2: hash += getshort (data);
				hash ^= hash << 11;
				hash += hash >> 17;
				break;
			case 1: hash += *data;
				hash ^= hash << 10;
				hash += hash >> 1;
		}

		/* Force "avalanching" of final 127 bits */
		hash ^= hash << 3;
		hash += hash >> 5;
		hash ^= hash << 2;
		hash += hash >> 15;
		hash ^= hash << 10;

		return hash;
	}

	HString *array_[ENTRIES];
};

typedef HashDb<256, 0xff> DbUser;
typedef HashDb<4096, 0xfff> DbDict;
//typedef HashDb<16384, 0x3fff> DbDict;

struct Dictionaries {
	public:
	Dictionaries(bool readmaster = false) {
		hptr_ = new DbDict();
		if (Global::instance()->lang_ro) {
			if (readmaster && !hptr_->getFile(DF1)) {
				LOG("Starting with an empty master dictionary");
			}
		}
		else if (Global::instance()->lang_en) {
			if (readmaster && !hptr_->getFile(DF3)) {
				LOG("Starting with an empty master dictionary");
			}
		}

		fptr_ = new DbUser();
		if (!fptr_->getFile(DF2)) {
			LOG("Starting with an empty fixed dictionary");
		}

		iptr_ = new DbUser();
		uptr_ = new DbUser();

	}

	~Dictionaries() {
		LOG("save dictionaries");
		// save new dictionary
#ifndef _WIN32
		char *homedir = getenv("HOME");
		if (homedir == 0) {
			fprintf(stderr, "Error: HOME environment variable nu a fost găsită\n");
			exit(1);
		}
		unsigned len = strlen(homedir) + strlen("/.rospell/dictionary.fixed");
		char *fname = new char[len + 1];
		memset(fname, '\0', len + 1);
		strcpy(fname, homedir);
		strcat(fname, "/.rospell/dictionary.fixed");
#else
		char *fname = "c:\\rospell\\dictionary.fixed";
#endif
		fptr_->putFile(fname);
#ifndef _WIN32
		delete[] fname;

		len = strlen(homedir) + strlen("/.rospell/dictionary.user");
		fname = new char[len + 1];
		memset(fname, '\0', len + 1);
		strcpy(fname, homedir);
		strcat(fname, "/.rospell/dictionary.user");
#else
		fname = "c:\\rospell\\dictionary.user";
#endif
		uptr_->putFile(fname);
#ifndef _WIN32
		delete[] fname;
#endif
		delete hptr_;
		delete fptr_;
		delete iptr_;
		delete uptr_;
	}

	DbDict *getMaster() {
		return hptr_;
	}
	void saveMaster() {
		ASSERT(hptr_ != 0);
		LOG("save master dictionary\n");
		if (Global::instance()->addallmaster)
			hptr_->putFile("dictionary.ro");
		else if (Global::instance()->addallmasterenglish)
			hptr_->putFile("dictionary.en");
		else
			ASSERT(0);
	}
	DbUser *getIgnore() {
		return iptr_;
	}
	DbUser *getUser() {
		return uptr_;
	}
	DbUser *getFixed() {
		return fptr_;
	}


	private:
	DbDict *hptr_;	// master dictionary
	DbUser *fptr_;	// fixed words dictionary
	DbUser *iptr_;	// ignore dictionary
	DbUser *uptr_;	// user dictionary
};

#endif
