//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************
//** $Id: qt/qwerty.cpp   3.3.4   edited May 27 2003 $
//**
//** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
//**
//** This file is part of an example program for Qt.  This example
//** program may be used, distributed and modified without limitation.
//**
//*****************************************************************************/

#include "qwerty.h"
#include <qapplication.h>
#include <qdesktopwidget.h>
#include <qdatetime.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <qmessagebox.h>
#include <qfontdialog.h>
#include <qtextcodec.h>
#include "busycursor.h"

#ifdef USING_QT4
#include <q3filedialog.h>
#include <q3popupmenu.h>
#include <q3paintdevicemetrics.h>
#include <q3ptrlist.h>
//Added by qt3to4:
#include <Q3CString>
#include <QCloseEvent>
#include <QResizeEvent>
#include <Q3VBoxLayout>
//Added by qt3to4:
#include <QLabel>
#endif
#ifdef USING_QT3
#include <qfiledialog.h>
#include <qpaintdevicemetrics.h>
#endif
#include <qfileinfo.h>
#include <qinputdialog.h>

#include "spell.h"
#include "global.h"
#include "utils.h"
#include "stats.h"
#include "dialogs.h"
#include "briefedit.h"
#include "lineandcolumn.h"
#include "ctagsdialog.h"
#ifdef _WIN32
#include "httpwindow.h"
#endif
const bool no_writing = FALSE;

Editor::Editor( QWidget * parent , const char * name )
: Q3VBox(parent, name)
//: QWidget( parent, name, Qt::WDestructiveClose )
{
	LOG("before resize in constructor");
	resize(Global::instance()->xsize, Global::instance()->ysize);
	LOG("after resize in constructor");

	pdict_ = new Dictionaries(true);
	readHomeDir(pdict_);
	search_ = "";
	default_format_ = FILE_UTF8;
	*word1_ = '\0';
	*word2_ = '\0';
	*word3_ = '\0';
	*word4_ = '\0';
	*word5_ = '\0';
	resize(Global::instance()->xsize, Global::instance()->ysize);


	Q3Accel *a_ = new Q3Accel(this);
	// set bookmarks
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_1), this, SLOT(setBookmark1()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_2), this, SLOT(setBookmark2()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_3), this, SLOT(setBookmark3()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_4), this, SLOT(setBookmark4()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_5), this, SLOT(setBookmark5()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_6), this, SLOT(setBookmark6()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_7), this, SLOT(setBookmark7()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_8), this, SLOT(setBookmark8()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_9), this, SLOT(setBookmark9()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_0), this, SLOT(setBookmark0()));
	// user programable keys
	a_->connectItem(a_->insertItem(Qt::Key_F1), this, SLOT(userF1()));
	a_->connectItem(a_->insertItem(Qt::Key_F2), this, SLOT(userF2()));
	a_->connectItem(a_->insertItem(Qt::Key_F3), this, SLOT(userF3()));
	a_->connectItem(a_->insertItem(Qt::Key_F4), this, SLOT(userF4()));
	// debug
	a_->connectItem(a_->insertItem(Qt::Key_F10), this, SLOT(roDebug()));

	m_ = new QMenuBar( this, "menu" );

	Q3PopupMenu * file_ = new Q3PopupMenu(m_);
	Q_CHECK_PTR( file_ );
	m_->insertItem( "&File", file_ );


	file_->insertItem( "&New Window",   this, SLOT(newDoc()));
	file_->insertItem( "&New Buffer",   this, SLOT(newDocBuffer()));
	file_->insertItem( "&Open...",  this, SLOT(load()), Qt::ALT+Qt::Key_E);
	file_->insertItem( "Open &Location",  this, SLOT(openWwwLocation()));
	file_->insertItem( "&Save...",  this, SLOT(save()), Qt::ALT+Qt::Key_W);
	file_->insertItem( "Sa&ve As",  this, SLOT(saveAs()));
	file_->insertItem( "Save All",  this, SLOT(saveAll()));
	file_->insertItem( "Close",  this, SLOT(closeBuffer()), Qt::ALT+Qt::Key_D );
	file_->insertItem( "E&xit", this, SLOT(close()), Qt::ALT+Qt::Key_X);
	file_->insertSeparator();
#ifndef QT_NO_PRINTER
	file_->insertItem( "&Print...", this, SLOT(print()), Qt::CTRL+Qt::Key_P );
#endif


	Q3PopupMenu *edit_ = new Q3PopupMenu(m_);
	Q_CHECK_PTR(edit_);
	m_->insertItem("Edit", edit_);

	edit_->insertItem( "&Undo", this, SLOT(undo()), Qt::ALT+Qt::Key_U);
	edit_->insertItem( "Toggle insert/overwrite", this, SLOT(toggleOverwrite()));
	edit_->insertSeparator();
	edit_->insertItem( "Cut", this, SLOT(cut()), Qt::CTRL+Qt::Key_X);
	edit_->insertItem( "Copy", this, SLOT(copy()), Qt::CTRL+Qt::Key_C);
	edit_->insertItem( "Paste", this, SLOT(paste()), Qt::CTRL+Qt::Key_V);
	edit_->insertItem( "Delete", this, SLOT(e_delete()));
	edit_->insertItem( "Select All", this, SLOT(selectAll()));
	edit_->insertSeparator();
	edit_->insertItem( "&Find", this, SLOT(search()), Qt::Key_F5);
	edit_->insertItem( "Find Ne&xt", this, SLOT(searchAgain()), Qt::SHIFT+Qt::Key_F5);
	edit_->insertItem( "Replace", this, SLOT(replace()), Qt::Key_F6);
	edit_->insertItem( "Grep", this, SLOT(grep()), Qt::Key_F7);
	edit_->insertSeparator();
	edit_->insertItem( "Goto &Line", this, SLOT(gotoLine()), Qt::ALT+Qt::Key_G);
	edit_->insertItem( "Start of File", this, SLOT(startOfBuffer()), Qt::CTRL+Qt::Key_PageUp);
	edit_->insertItem( "End of File", this, SLOT(endOfBuffer()), Qt::CTRL+Qt::Key_PageDown);
	edit_->insertItem( "Set &Bookmark", this, SLOT(setBookmark()));
	edit_->insertItem( "Goto Boo&kmark", this, SLOT(gotoBookmark()), Qt::ALT+Qt::Key_J);

	Q3PopupMenu * config_ = new Q3PopupMenu(m_);
	Q_CHECK_PTR( config_ );
	m_->insertItem( "&Options", config_ );

	config_->insertItem("Select &Font", this, SLOT(font()));
	config_->insertItem("&Colors", this, SLOT(colors()));
	config_->insertItem("&Alias", this, SLOT(editAlias()));
	config_->insertItem("C&tags", this, SLOT(configCtags()));
	config_->insertItem("&Language", this, SLOT(configLanguage()));
	config_->insertSeparator();
	config_->insertItem("S&tatus", this, SLOT(status()));


	buffers_ = new Q3PopupMenu(m_);
	Q_CHECK_PTR(buffers_);
	m_->insertItem("&Buffers", buffers_);
	connect(buffers_, SIGNAL(activated(int)), this, SLOT(pickBuffer(int)));
	buffers_->insertItem("Next", this, SLOT(nextBuffer()), Qt::ALT+Qt::Key_N);
	buffers_->insertSeparator();
     
	Q3PopupMenu * tools_ = new Q3PopupMenu(m_);
	Q_CHECK_PTR( tools_ );
	m_->insertItem( "T&ools", tools_ );

	tools_->insertItem( "&Record" , this, SLOT(recordAction()), Qt::ALT+Qt::Key_R);
	tools_->insertItem( "&Playback" , this, SLOT(playbackAction()), Qt::ALT+Qt::Key_P);
	tools_->insertSeparator();
	tools_->insertItem( "&Uppercase" ,   this, SLOT(uppercase()));
	tools_->insertItem( "&Lowercase" ,   this, SLOT(lowercase()));
	tools_->insertItem( "&dexonline.ro" ,   this, SLOT(dexonline()));
	tools_->insertItem( "&UNIX CR/LF cleanup" ,   this, SLOT(unixCleanup()));
	tools_->insertItem( "&Diff" ,   this, SLOT(diff()));
	tools_->insertItem( "&Session Diff" ,   this, SLOT(sessionDiff()));


	// words spelling menubar
//	Q3VBoxLayout *vly = new Q3VBoxLayout(this);
//	vly->setAutoAdd(true);
	Q3VBox *vbox = new Q3VBox(this);
	mwords_ = new QMenuBar(vbox, "words menu" );
#ifdef _WIN32
//	QFont courier("Courier New",16,50); courier.setPixelSize(16);
	QFont courier("Courier",9);
	mwords_->setFont(courier);
#endif

	mwords_->insertItem( "Start Spelling",   this, SLOT(spellAdvance()));
	mwords_->insertItem( "Ignore",   this, SLOT(spellIgnore()));
	mwords_->insertItem( "Add",   this, SLOT(spellAdd()));
	if (Global::instance()->fixeddict)
		mwords_->insertItem( "Add na&me",   this, SLOT(spellName()),   Qt::ALT+Qt::Key_M );
	mw1_ = mwords_->insertItem( " ",   this, SLOT(replaceWord1()), 0 );
	mw2_ = mwords_->insertItem( " ",   this, SLOT(replaceWord2()), 0 );
	mw3_ = mwords_->insertItem( " ",   this, SLOT(replaceWord3()), 0 );
	mw4_ = mwords_->insertItem( " ",   this, SLOT(replaceWord4()), 0 );
	mw5_ = mwords_->insertItem( " ",   this, SLOT(replaceWord5()), 0 );



	changed_ = FALSE;
//	e_ = new Q3MultiLineEdit(vbox, "editor" );
	e_ = new BriefEdit(vbox, "editor" );
	// get default XML syntax
	syntax_ = getSyntax("", e_);
	connect( e_, SIGNAL( textChanged() ), this, SLOT( textChanged() ) );
	connect( e_, SIGNAL( clicked(int, int) ), this, SLOT( clicked(int, int) ) );
#ifdef _WIN32
	e_->setFont(courier);
#endif

	e_->setWordWrap(Q3TextEdit::NoWrap);

	// colors
	e_->setPaletteBackgroundColor(Global::instance()->c_background);
	e_->setPaletteForegroundColor(Global::instance()->c_foreground);
	QPalette qp = e_->palette();
	qp.setColor(QPalette::Active, QColorGroup::Highlight, Global::instance()->c_highlight);
	qp.setColor(QPalette::Inactive, QColorGroup::Highlight, Global::instance()->c_highlight);
	e_->setPalette(qp);
	// user area
	tw_ = new QTabWidget(vbox);
	tw_->setMaximumHeight(200);
	tw_user_ = new Q3MultiLineEdit(vbox);
	tw_user_->setReadOnly(TRUE);
	tw_user_->setPaletteBackgroundColor(Global::instance()->c_background);
	tw_user_->setPaletteForegroundColor(Global::instance()->c_foreground);
	tw_user_->setPalette(qp);
	tw_search_ = new Q3MultiLineEdit(vbox);
	tw_search_->setReadOnly(TRUE);
	tw_search_->setPaletteBackgroundColor(Global::instance()->c_background);
	tw_search_->setPaletteForegroundColor(Global::instance()->c_foreground);
	tw_search_->setPalette(qp);
	tw_tags_ = new Q3MultiLineEdit(vbox);
	tw_tags_->setReadOnly(TRUE);
	tw_tags_->setPaletteBackgroundColor(Global::instance()->c_background);
	tw_tags_->setPaletteForegroundColor(Global::instance()->c_foreground);
	tw_tags_->setPalette(qp);
	tw_tags_syntax_ = getSyntax("file.cpp", tw_tags_);
#ifdef _WIN32
	tw_tags_->setFont(courier);
#endif
	connect(tw_tags_, SIGNAL(clicked(int, int)), this, SLOT(clickedTags(int, int)));
	connect(tw_search_, SIGNAL(clicked(int, int)), this, SLOT(clickedSearch(int, int)));
	connect(tw_user_, SIGNAL(clicked(int, int)), this, SLOT(clickedUser(int, int)));

	tw_->addTab(tw_tags_, "tags");
	tw_->addTab(tw_search_, "search");
	tw_->addTab(tw_user_, "user");


	status_ = new QStatusBar(vbox, "status bar");
	connect( e_, SIGNAL(cursorPositionChanged(int, int)), this, SLOT(cursorPosChanged(int, int)));
	lac_ = new LineAndColumn(status_, "line and column widget");
	status_->addWidget(lac_, 0, TRUE);

	// set keyboard focus in the editor window
	e_->setFocus();

	// connect ctags thread
//#ifdef USING_QT4
	thread_ = new CtagsThread(this);
	thread_->start(QThread::LowPriority);
#ifdef USING_QT4
	connect((const QObject *) thread_, SIGNAL(alldone()), this, SLOT(ctagsDone()));
#endif
}

Editor::~Editor()
{
	LOG("Editor::~Editor");
}

void Editor::releaseAll() {
	LOG("releaseAll");
	delete syntax_;
	delete tw_tags_syntax_;
	delete pdict_;
	delete m_;
	delete mwords_;
	delete e_;
	// terminate the thread
	thread_->terminate();
	thread_->wait();
	delete thread_;
	delete status_;
}

void Editor::recordAction() {
	e_->recordAction();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	cursorPosChanged(para, index);
}

void Editor::playbackAction() {
	e_->playbackAction();
}

void Editor::toggleOverwrite() {
	bool status = e_->isOverwriteMode();
LOG("%d overwrite");
	e_->setOverwriteMode(!status);
}

void Editor::cursorPosChanged(int para, int pos) {
	char msg[500];
	if (e_->isRecording())
		sprintf(msg, "Recording: %d/%d", para + 1, pos + 1);
	else
		sprintf(msg, "%d/%d", para + 1, pos + 1);
	lac_->setText(msg);

	if (!thread_->isActive())
		return;
	QString line = e_->textLine(para);
	QChar ch = line.at(pos);
	if (ch.isLetterOrNumber() || ch == '_') {
		// extract word
		int xstart = pos;
		while (xstart >= 0 && (line.at(xstart).isLetterOrNumber() || line.at(xstart) == '_'))
			xstart--;
		if (xstart != 0)
			xstart++;

		int xend = pos;
		while (line.at(xend).isLetterOrNumber() || line.at(xend) == '_')
			xend++;

		QString ctagword = line.mid(xstart, xend - xstart);
		if (ctagword.startsWith("\t"))
			ctagword = ctagword.mid(1);
		thread_->startProcessing(ctagword);
	}
}

void Editor::pickBuffer(int id) {
	if (id < 0)
		return;
	saveCurentBuffer();

	int index = id - 1;
LOG("got alt key, id %d", id);
	// move to the specific buffer
	b_.setActiveIndex(index);
	
	// restore data
	changed_ = TRUE;
	QString fname = b_.getActiveFName();
	setFname(fname, false);
	e_->setText(b_.getActiveText());
	
	// some 50ms delay here
	QTime t;
	t.start();
	while (t.elapsed() < 50) {
		qApp->processEvents();
	}
	
	if (fname.startsWith("*"))
		changed_ = TRUE;
	else
		changed_ = FALSE;
	int line = b_.getActiveLine();
	int col = b_.getActiveCol();
//LOG("get line %d col %d", line, col);
	e_->setCursorPosition(line, col, false);

//LOG("starting redo");	
	redoBufMgrList();



}

void Editor::saveCurentBuffer() {
	b_.setActiveText(e_->text());
	b_.setActiveFName(caption());
	int line;
	int col;
	e_->getCursorPosition(&line, &col);
//LOG("save line %d col %d", line, col);
	b_.setActiveLine(line);
	b_.setActiveCol(col);
}

void Editor::nextBuffer() {
	// save curent text first
	saveCurentBuffer();
	
	// move to the next buffer
	b_.setActiveNext();

	
	// restore data
	changed_ = TRUE;
	QString fname = b_.getActiveFName();
	setFname(fname, false);
	e_->setText(b_.getActiveText());	
	if (fname.startsWith("*"))
		changed_ = TRUE;
	else
		changed_ = FALSE;

	int line = b_.getActiveLine();
	int col = b_.getActiveCol();
//LOG("get line %d col %d", line, col);
	e_->setCursorPosition(line, col, false);
	syntax_->rehighlight();

//LOG("starting redo");	
	redoBufMgrList();
}


bool Editor::saveAll() {
	for (int i = 0; i < MAXBUFMGR; i++) {
		QString fname = b_.getFName(i);
		if (fname.isNull())
			break;
		if (fname.startsWith("*")) {
			saveCommon(fname.mid(1), b_.getFormat(i));
			b_.setFName(fname.mid(1), i);
		}
	}
	redoBufMgrList();
	return true;
}


void Editor::redoBufMgrList() {
	buffers_->clear();
	buffers_->insertItem( "&Next", this, SLOT(nextBuffer()), Qt::ALT+Qt::Key_N);
	buffers_->insertSeparator();
	for (int i = 0; i < MAXBUFMGR; i++) {
		QString fname = b_.getFName(i);
		if (fname.isNull())
			break;
		buffers_->insertItem(fname, i + 1);
	}
}




void Editor::setFname(QString fname, bool modified) {
	if (modified) {
		QString new_fname = "*";
		new_fname.append(fname);
		setCaption(new_fname);
	}
	else {
		setCaption(fname);
	}

	// set correct syntax highlighter		
	if (syntax_ != 0)
		delete syntax_;
	syntax_ = getSyntax(fname, e_);
}

QString Editor::getFname() {
	QString retval = caption();

	if (retval.startsWith("*"))
		retval = retval.mid(1);
	
	return retval;
}


#ifdef _WIN32
void Editor::dexonline() {
	QString sword = e_->markedText();

	if (sword.isNull()) {
		LOG("dexonline no word");
		return;
	}
	Q3CString utext = sword.utf8();
	const char *cstr = utext;
	LOG("Editor::dexonline  %s", cstr);
	msgScreen("Loading...");
	QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );

	char cmd[1000];
	snprintf(cmd, 1000, "http://dexonline.ro/search.php?cuv=%s&source=", cstr);
	HttpWindow httpWin(this, cmd);
	httpWin.show();
	httpWin.exec();
	char *www_content = UTILS::read_file_malloc((const char *) httpWin.getFname());
	LOG("%s\n\n", www_content);
//LOG("www: %s", www_content);
	msgScreen(0);
	QApplication::restoreOverrideCursor();
	if (www_content != 0) {
//		FILE_FORMAT format = detectWwwFormat(www_content);
//		QString txt = toUtf8(www_content, format);
		char *newcontent = stripDexonline(www_content);
		if (newcontent == 0) {
			delete[] www_content;
			return;
		}
		LOG("******\n%s\n***************\n", newcontent);
//		QMessageBox::warning(0, cstr, txt);
		QMessageBox::warning(0, "http://dexonline.ro", QString::fromUtf8(newcontent));
		delete[] newcontent;
	}
}
#else //!_WIN32

void Editor::dexonline() {
	// check for wget presence
	if (!Global::checkWhich("wget"))
		return;

	QString sword = e_->markedText();
	if (sword.isNull()) {
		LOG("dexonline no word");
		return;
	}
	Q3CString utext = sword.utf8();
	const char *cstr = utext;
	LOG("Editor::dexonline  %s", cstr);
	msgScreen("Loading...");
	QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );

	char cmd[1000];
	snprintf(cmd, 1000, "wget --timeout=60 --tries=1 -q --output-document=- http://dexonline.ro/search.php?cuv=%s&source=", cstr);
	char *www_content = UTILS::run_prg_malloc(cmd);
	msgScreen(0);
	QApplication::restoreOverrideCursor();
	if (www_content != 0) {
		char *newcontent = stripDexonline(www_content);
		if (newcontent == 0) {
			delete[] www_content;
			return;
		}
		LOG("******\n%s\n***************\n", newcontent);
		QMessageBox::about(0, "http://dexonline.ro", QString::fromUtf8(newcontent));
		delete[] newcontent;
	}
}
#endif //_WIN32

static void cleanTags(char *str, char *tag1, char *tag2) {
	char *ptr1 = strstr(str, tag1);
	if (ptr1 == 0)
		return;
	char *ptr3 = strstr(ptr1, tag2);
	if (ptr3 == 0)
		return;
	ptr3 += strlen(tag2);

	if ((unsigned) ptr1 < (unsigned) ptr3) {
		while((unsigned) ptr1 != (unsigned) ptr3) {
			*ptr1 = ' ';
			ptr1++;
		}
	}
}

char *Editor::stripDexonline(char *str) {
	ASSERT(str != 0);
	if (str == 0)
		return 0;

	char *ptr1 = UTILS::buffer_sed(str, "&#x103;", "ă");
	delete[] str;
	char *ptr2 = UTILS::buffer_sed(ptr1, "&#xee;", "î");
	delete[] ptr1;
	ptr1 = UTILS::buffer_sed(ptr2, "&#xe2;", "â");
	delete[] ptr2;
	ptr2 = UTILS::buffer_sed(ptr1, "&#x2013;", "-");
	delete[] ptr1;
	ptr1 = UTILS::buffer_sed(ptr2, "&#xcd;", "I");
	delete[] ptr2;
	ptr2 = UTILS::buffer_sed(ptr1, "&#x102;", "Ă");
	delete[] ptr1;
	ptr1 = UTILS::buffer_sed(ptr2, "&#xc2;", "Â");
	delete[] ptr2;
	ptr2 = UTILS::buffer_sed(ptr1, "&#xc9;", "E");
	delete[] ptr1;
	ptr1 = UTILS::buffer_sed(ptr2, "&#xce;", "Î");
	delete[] ptr2;
	ptr2 = UTILS::buffer_sed(ptr1, "&#xd3;", "O");
	delete[] ptr1;
	ptr1 = UTILS::buffer_sed(ptr2, "&#xda;", "U");
	delete[] ptr2;
	ptr2 = UTILS::buffer_sed(ptr1, "&#xc1;", "A");
	delete[] ptr1;

	cleanTags(ptr2, "<script", "</script>");
	cleanTags(ptr2, "<script", "</script>");
	cleanTags(ptr2, "<script", "</script>");
	cleanTags(ptr2, "<script", "</script>");
	cleanTags(ptr2, "<script", "</script>");
	cleanTags(ptr2, "<script", "</script>");
	cleanTags(ptr2, "<noscript", "</noscript>");
	cleanTags(ptr2, "<form", "</form>");
	return ptr2;
}


void Editor::unixCleanup() {
	QString text = e_->text();	
	Q3CString utext = text.utf8();
	const char *ctext = utext;
	char *output1 = UTILS::buffer_sed((char *)ctext, "\n\r", "\n", true);
	char *output2 = UTILS::buffer_sed((char *)output1, "\r\n", "\n", true);
	delete[] output1;
	QString final = QString::fromUtf8(output2);
	e_->setText(final);
	delete[] output2;
}



void Editor::uppercase() {
	QString text = e_->markedText();
	Q3CString utext = text.utf8();
	const char *ctext = utext;
	char *word = UTF8::strtoupper(ctext);
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	delete word;
}

void Editor::lowercase() {
	QString text = e_->markedText();
	Q3CString utext = text.utf8();
	const char *ctext = utext;
	char *word = UTF8::strtolower(ctext);
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	delete word;
}

char *Editor::format2str(FILE_FORMAT format) {
	char *sformat = "unknown";
	switch (format) {
		case FILE_UTF8:
			sformat = "Unicode UTF-8";
			break;

		case FILE_ISO8859_2:
			sformat = "ISO8859-2 (Latin 2)";
			break;

		case FILE_ISO8859_16:
			sformat = "ISO8859-16 (South-Eastern European)";
			break;

		case FILE_CP1250:
			sformat = "CP1250 (Microsoft Latin II)";
			break;

		default:
			ASSERT(0);
			break;
	}
	return sformat;
}

void Editor::status() {
	char str[1000];

	QString fname = getFname();
	Q3CString ufname = fname.utf8();
	const char *csname = ufname;

	char *sformat = format2str(default_format_);

	int lines = e_->numLines();

	char speller_stats[100];
	Stats::instance()->sprint(speller_stats, 100);

	QFont fnt = e_->font();
	const char *ffamily = (const char *) fnt.family().ascii();

	int para;
	int index;
	e_->getCursorPosition(&para, &index);

	snprintf(str, 1000, "\n    rospell version %s    \n\n    File: %s\n    Loaded as: %s\n"
		    "    Lines: %d\n\n"
		    "    Font family: %s    \n"
		    "    Font test: ăşâţîĂŞÂŢÎ    \n"
		    "\n%s    Dictionary: %s\n\n"
		    "    %d ctags\n"
		    "    Cursor: %d/%d\n",
		    VERSION, csname, sformat, lines, ffamily, speller_stats,
		    (Global::instance()->lang_en)? "English": "Romanian",
		    thread_->getTagsNumber(), para + 1, index + 1);

	StatusDialog *sd = new StatusDialog(fnt, str);
	sd->setCaption("File status");
	sd->exec();
//	QMessageBox::warning(0,"File status", str);
	delete sd;
}

void Editor::font()
{
	bool ok;
	QFont f = QFontDialog::getFont( &ok, e_->font() );
	if (ok) {
		e_->setFont( f );
		LOG("set font %s", (const char *) (f.family()).utf8());
		mwords_->setFont(f);
		mwords_->clear();
		mwords_->insertItem( "Start Spelling",   this, SLOT(spellAdvance()));
		mwords_->insertItem( "Ignore",   this, SLOT(spellIgnore()));
		mwords_->insertItem( "Add",   this, SLOT(spellAdd()));
		if (Global::instance()->fixeddict)
			mwords_->insertItem( "Add na&me",   this, SLOT(spellName()),   Qt::ALT+Qt::Key_M );
		mw1_ = mwords_->insertItem( " ",   this, SLOT(replaceWord1()), 0 );
		mw2_ = mwords_->insertItem( " ",   this, SLOT(replaceWord2()), 0 );
		mw3_ = mwords_->insertItem( " ",   this, SLOT(replaceWord3()), 0 );
		mw4_ = mwords_->insertItem( " ",   this, SLOT(replaceWord4()), 0 );
		mw5_ = mwords_->insertItem( " ",   this, SLOT(replaceWord5()), 0 );

	}
}

void Editor::newDoc()
{
	LOG("newDoc");
	Editor *ed = new Editor;
	if (qApp->desktop()->size().width() < 450
	    || qApp->desktop()->size().height() < 450) {
		ed->showMaximized();
	} else {
		ed->resize(Global::instance()->xsize, Global::instance()->ysize);
		LOG("Editor::newDoc sizes %d %d", Global::instance()->xsize, Global::instance()->ysize);
		ed->show();
	}
}

void Editor::newDocBuffer()
{
	LOG("newDocBuffer");
	// save curent text first
	saveCurentBuffer();

	// add new empty buffer
	changed_ = TRUE;
	b_.add("", default_format_);
	setFname("");
	e_->setText("");
	e_->setCursorPosition(1, 1, false);
	redoBufMgrList();
	changed_ = FALSE;
}


FILE_FORMAT Editor::detectWwwFormat(const char *str) {
	ASSERT(str != 0);
	FILE_FORMAT retval = FILE_UTF8;

	// detect iso8859-2
	while (1) {
		char *ptr = strcasestr(str, "<head>");
		if (ptr == 0)
			break;
		ptr = strcasestr(ptr, "charset=iso-8859-2");
		if (ptr == 0)
			break;
		ptr = strcasestr(ptr, "</head>");
		if (ptr == 0)
			break;
		retval = FILE_ISO8859_2;
		break;
	}

	// detect win1250
	while (1) {
		char *ptr = strcasestr(str, "<head>");
		if (ptr == 0)
			break;
		ptr = strcasestr(ptr, "charset=windows-1250");
		if (ptr == 0)
			break;
		ptr = strcasestr(ptr, "</head>");
		if (ptr == 0)
			break;
		retval = FILE_CP1250;
		break;
	}

	return retval;
}

QString Editor::toUtf8(const char *str, FILE_FORMAT format) {
	QString retval;

	switch (format) {
		case FILE_UTF8: {
				retval = QString::fromUtf8(str);
				LOG("Editor::toUtf8 utf8");
			}
			break;
		case FILE_ISO8859_2: {                
				QTextCodec *codec = QTextCodec::codecForName("ISO8859-2");
				QTextDecoder *decoder = codec->makeDecoder();
				retval = decoder->toUnicode(str, strlen(str));
				LOG("Editor::toUtf8 iso8859-2");
			}
			break;
		case FILE_ISO8859_16: {                
				QTextCodec *codec = QTextCodec::codecForName("ISO8859-16");
				QTextDecoder *decoder = codec->makeDecoder();
				retval = decoder->toUnicode(str, strlen(str));
				LOG("Editor::toUtf8 iso8859-16");
			}
			break;
		case FILE_CP1250: {
				QTextCodec *codec = QTextCodec::codecForName("CP1250");
				QTextDecoder *decoder = codec->makeDecoder();
				retval = decoder->toUnicode(str, strlen(str));
				LOG("Editor::toUtf8 cp1250");
			}
			break;
		default:
			ASSERT(0);
	}

	return retval;
}

#ifdef _WIN32
void Editor::openWwwLocation()
{
	if (changed_) { // the text has been changed
		switch (QMessageBox::warning( this, "Question",
								"Save changes to Document?",
								tr("&Yes"),
								tr("&No"),
								tr("Cancel"),
								0, 2)) {
			case 0: // yes
				save();
				break;
			case 1: // no
				break;
			default: // cancel
				return;
				break;
		}
	}

	msgScreen("Loading...");

	HttpWindow httpWin;
	httpWin.show();
	httpWin.exec();
	char *www_content = UTILS::read_file_malloc((const char *) httpWin.getFname());

	if (www_content != 0) {
		FILE_FORMAT format = detectWwwFormat(www_content);
		QString txt = toUtf8(www_content, format);
		e_->setText(txt);
//		sword.replace(QChar('/'), "_");
		setFname(httpWin.getFname());
//lucian: not implemented
		default_format_ = format; //lucian: do not set default format!!!
		msgScreen(format2str(format), 2000);
		msgScreen(0);
//		QMessageBox::warning(0,"cstr", txt);
		delete[] www_content;
		changed_ = false;
		b_.add(httpWin.getFname(), format);
		b_.setActiveText(txt);
		int line;
		int col;
		e_->getCursorPosition(&line, &col);
		b_.setActiveLine(line);
		b_.setActiveCol(col);
		e_->setCursorPosition(1, 1, false);

		redoBufMgrList();
	}
}
#else //!_WIN32
void Editor::openWwwLocation()
{
	if (changed_) { // the text has been changed
		switch (QMessageBox::warning( this, "Question",
								"Save changes to Document?",
								tr("&Yes"),
								tr("&No"),
								tr("Cancel"),
								0, 2)) {
			case 0: // yes
				save();
				break;
			case 1: // no
				break;
			default: // cancel
				return;
				break;
		}
	}

	// check for wget presence
	if (!Global::checkWhich("wget"))
		return;

	WwwDialog *wd = new WwwDialog();
	wd->setCaption("Enter document location");
	wd->exec();
	QString sword = wd->getWord();

	if (sword == "")
		return;
	Q3CString utext = sword.utf8();
	const char *cstr = utext;
	LOG("Editor::openWwwLocation  %s", cstr);
	delete wd;
	msgScreen("Loading...");
	QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );

	char cmd[1000];
	snprintf(cmd, 1000, "wget --timeout=60 --tries=1 -q --output-document=- %s", cstr);
	char *www_content = UTILS::run_prg_malloc(cmd);
	QApplication::restoreOverrideCursor();
	if (www_content != 0) {
		FILE_FORMAT format = detectWwwFormat(www_content);
		QString txt = toUtf8(www_content, format);
		e_->setText(txt);
		sword.replace(QChar('/'), "_");
		setFname(sword);
		default_format_ = format; //lucian: do not set default format!!!
		msgScreen(format2str(format), 2000); //lucian: somehow doesn't work
		msgScreen(0);
		delete[] www_content;
		changed_ = false;
		b_.add(sword, format);
		b_.setActiveText(txt);
		int line;
		int col;
		e_->getCursorPosition(&line, &col);
		b_.setActiveLine(line);
		b_.setActiveCol(col);
		e_->setCursorPosition(1, 1, false);

		redoBufMgrList();
		msgScreen("File loaded.");
	}
	else {
		msgScreen("Failed to load the file.");
	}
}
#endif

void Editor::msgScreen(const char *str, int mstime) {
	status_->message(str, mstime);
// old version using the menu bar
//	ASSERT(time >= 0);
//	if (str == 0)
//		m_->changeItem(msgid_, QString("          "));
//	else
//		m_->changeItem(msgid_, QString(str));
//
//	LOG("mstime %d", mstime);
//	if (mstime == 0) {
//		qApp->processEvents();
//		return;
//	}
//
//	QTime t;
//	t.start();
//	while (t.elapsed() < mstime) {
//		qApp->processEvents();
//	}
}



void Editor::setDefaultFormat(FILE_FORMAT format) {
	default_format_ = format;
}

void Editor::load()
{
	msgScreen("loading file, please wait...");
	QString fn = Q3FileDialog::getOpenFileName( QString::null, QString::null, this );
	if (!fn.isEmpty())
		load(fn);
	else
		msgScreen("aborted");
}


void Editor::load(const QString& fileName)
{
	BCursor busy;
	msgScreen("loading file, please wait...");
	// save curent buffer first
	saveCurentBuffer();

	// move to an absolute file name
	QFileInfo qf(fileName);
	QString newfileName = qf.absFilePath();

	// check if the file is already loaded
	bool foundit = false;
	int index = 0;
	for (index = 0; index < MAXBUFMGR; index++) {
		QString fname = b_.getFName(index);
		if (fname.isNull())
			break;
		if (fname.startsWith("*")) {
			fname = fname.mid(1);
		}
		if (fname == newfileName) {
LOG("found file at id %d", index);
			foundit = true;
			break;
		}
	}
	if (foundit) {
		pickBuffer(index + 1);
		return;
	}

	changed_ = true;
	Q3CString qfname = newfileName.utf8();
	const char *fname = qfname;
	LOG("Editor::load %s %d", fname, (int) default_format_);
	e_->setAutoUpdate( FALSE );
	setFname(newfileName, false);

	QFile f( fileName );
#ifdef USING_QT3
	if (f.open(IO_ReadOnly )) {
#endif
#ifdef USING_QT4
	if (f.open( QIODevice::ReadOnly )) {
#endif

		QTextStream t(&f);
		switch (default_format_) {
			case FILE_UTF8:
				t.setEncoding( QTextStream::UnicodeUTF8 );
				LOG("Editor::load utf8");
				break;
			case FILE_CP1250: {
					// lucian: qt has some problems with this conversion!!!                
					QTextCodec *codec = QTextCodec::codecForName("CP1250");
					t.setCodec(codec);
					LOG("Editor::load cp1250");
				}
				break;
			case FILE_ISO8859_2: {
					QTextCodec *codec = QTextCodec::codecForName("ISO8859-2");
					t.setCodec(codec);
					LOG("Editor::load iso8859-2");
				}
				break;
			case FILE_ISO8859_16: {
					QTextCodec *codec = QTextCodec::codecForName("ISO8859-16");
					t.setCodec(codec);
					LOG("Editor::load iso8859-16");
				}
				break;
			default:
				ASSERT(0);
				msgScreen("encoding not found; file not loaded");
				return;
		}
	
		e_->setText( t.read() );
		f.close();
	} // if file open
	else
		e_->setText("");

	// backup
	QString fname_only = qf.fileName();
	char *backup_fname = Global::getBackupFname((const char *) qf.fileName());
	saveCommon(QString(backup_fname), default_format_, true);
	delete[] backup_fname;

	e_->setAutoUpdate( TRUE );
	e_->repaint();
	changed_ = FALSE;
	cleanMWords();
	
	// set buffer
	b_.add(newfileName, default_format_);
	b_.setActiveText(e_->text());
	e_->setCursorPosition(0, 0, false);
	redoBufMgrList();
	msgScreen("file loaded");
}


void Editor::save()
{
	msgScreen("saving file, please wait...");
	QString fname = getFname();
	if (fname.isNull())
		saveAs();
	else {
		BCursor busy;
		Q3CString utext = fname.utf8();
		const char *cstr = utext;
		LOG("Editor::save file %s", cstr);
		if (!fname.isEmpty())
			saveCommon(fname, default_format_);
		else
			msgScreen("aborted");
	}
	msgScreen("file saved");
	e_->update();
}

bool Editor::saveAs()
{
	msgScreen("saving file, please wait...");
	// name change in bufmgr is done in saveComon
	
	//storing filename (proper save) is left as an exercise...
	QString fn = Q3FileDialog::getSaveFileName( QString::null, QString::null, this );
	BCursor busy;
	if (!fn.isEmpty())
		return saveCommon(fn, default_format_);
	msgScreen("aborted");
	return FALSE;
}



bool Editor::saveCommon(const QString& fileName, FILE_FORMAT format, bool simple_save)
{
	QFile f( fileName );
	
#ifdef USING_QT3
	if (!f.open(IO_WriteOnly )) {
#endif
#ifdef USING_QT4
	if (!f.open( QIODevice::WriteOnly )) {
#endif

		QMessageBox::warning(this,"I/O Error",
						 QString("The file could not be opened.\n\n")
						 +fileName);
		msgScreen("file not saved");
		return FALSE;
	}
	QTextStream t(&f);
	LOG("save common format %d", format);

	switch (format) {
		case FILE_UTF8:
			t.setEncoding( QTextStream::UnicodeUTF8 );
			LOG("Editor::saveCommon utf8");
			break;
		case FILE_CP1250: {
				// lucian: qt has some problems with this conversion!!!                
				QTextCodec *codec = QTextCodec::codecForName("CP1250");
				t.setCodec(codec);
				LOG("Editor::saveCommon cp1250");
			}
			break;
		case FILE_ISO8859_2: {
				QTextCodec *codec = QTextCodec::codecForName("ISO8859-2");
				t.setCodec(codec);
				LOG("Editor::saveCommon iso8859-2");
			}
			break;
		case FILE_ISO8859_16: {
				QTextCodec *codec = QTextCodec::codecForName("ISO8859-16");
				t.setCodec(codec);
				LOG("Editor::saveCommon iso8859-16");
			}
			break;
		default:
			ASSERT(0);
			msgScreen("file not saved");
			return FALSE;
	}

	t << e_->text();
	f.close();
	if (simple_save)
		return TRUE;
		
	setFname(fileName);
	changed_ = FALSE;
	cleanMWords();
	b_.setActiveFName(fileName);
	b_.setActiveText(e_->text());
	int line;
	int col;
	e_->getCursorPosition(&line, &col);
	b_.setActiveLine(line);
	b_.setActiveCol(col);
	redoBufMgrList();
	msgScreen("file saved");
	e_->setFocus();
	
	
	return TRUE;
}



void Editor::print()
{
#ifndef QT_NO_PRINTER
	if (printer_.setup(this)) {			   // opens printer dialog
		printer_.setFullPage(TRUE);			 // we'll set our own margins
		QPainter p;
		p.begin( &printer_ );				 // paint on printer
		p.setFont( e_->font() );
		QFontMetrics fm = p.fontMetrics();
		Q3PaintDeviceMetrics metrics( &printer_ ); // need width/height
		// of printer surface
		const int MARGIN = metrics.logicalDpiX() / 2; // half-inch margin
		int yPos        = MARGIN;			// y position for each line

		for (int i = 0 ; i < e_->numLines() ; i++) {
			if (printer_.aborted())
				break;
			if (yPos + fm.lineSpacing() > metrics.height() - MARGIN) {
				// no more room on this page
				if (!printer_.newPage())		    // start new page
					break;					   // some error
				yPos = MARGIN;				   // back to top of page
			}
			p.drawText( MARGIN, yPos, metrics.width() - 2*MARGIN,
					  fm.lineSpacing(), ExpandTabs, e_->textLine( i ) );
			yPos += fm.lineSpacing();
		}
		p.end();							// send job to printer
	}
	cleanMWords();
#endif
}

void Editor::resizeEvent( QResizeEvent * )
{
	LOG("resizeEvent sizes %d %d", width(), height());
//	if (e_ && m_ )
//		e_->setGeometry( 0, m_->height(), width(), height() - m_->height());
	Global::instance()->xsize = width();
	Global::instance()->ysize = height();
}

void Editor::closeEvent( QCloseEvent *event )
{
	LOG("closeEvent");
	event->accept();
	bool torelease = true;
	
	if (changed_ || b_.anyBufChanged()) { // the text has been changed
		switch (QMessageBox::warning( this, "Question",
			"One or more buffers have not been saved. Save all?",
			tr("&Yes"),
			tr("&No"),
			tr("Cancel"),
			0, 2)) {
			case 0: // yes
				if (saveAll())
					event->accept();
				else {
					event->ignore();
					torelease = false;
				}
				break;
			case 1: // no
				event->accept();
				break;
			default: // cancel
				event->ignore();
				torelease = false;
				break;
		}
	}
	
	if (torelease) {
		releaseAll();
	}
}

void Editor::closeBuffer()
{
	LOG("closeBuffer");
	// save curent buffer first
	saveCurentBuffer();

	if (changed_) { // the text has been changed
		switch (QMessageBox::warning( this, "Question",
								"Save changes to Document?",
								tr("&Yes"),
								tr("&No"),
								tr("Cancel"),
								0, 2)) {
			case 0: // yes
				save();
				break;
			case 1: // no
			default: // cancel
				break;
		}
	}
	
	// first move some active pointers
	e_->setText("");
	setFname("rospell", false);

	// delete buffer
	int aindex = b_.getActiveIndex();
	b_.del(aindex);
		
	// restore data
	changed_ = TRUE;
	e_->setText(b_.getActiveText());	
	QString fname = b_.getActiveFName();
	setFname(fname, false);
	if (fname.startsWith("*"))
		changed_ = TRUE;
	else
		changed_ = FALSE;
	redoBufMgrList();
	int line = b_.getActiveLine();
	int col = b_.getActiveCol();
	e_->setCursorPosition(line, col, false);
	syntax_->rehighlight();
}



void Editor::advanceStartWord() {

//	LOG("enter advanceStartWord function with index %d", x_);
	int lines = e_->numLines();
//	LOG("lines %d", lines);

	while (1) {
		QChar ch = line_.at(x_);
		for (int i = 0; !ch.isNull(); i++, x_++) {
			ch = line_.at(x_);
			if (ch.isLetterOrNumber() || ch == '<' || ch == '/' || ch == '&') { // check for utf8 also
//				LOG("return advanceStartWord function with index %d", x_);
				return;
			}
		}

		if ((y_ + 1) >= lines) {
//			LOG("advanceStartWord advance end of file detected");
			e_->setCursorPosition(y_, x_, false);
			break;
		}

		y_++;
		x_ = 0;
		line_ = e_->textLine(y_);
		e_->setCursorPosition(y_, x_, false);
//		LOG("advanceStartWord advance para %d, index %d", y_, x_);
	}
}

QString Editor::advanceEndWord() {
	if (e_->atEnd())
		return QString("");

	xword_ = x_;
//	LOG("enter advanceEndWord with index %d", x_);

	QChar ch = line_.at(x_);
	int i;
	for (i = 0; !ch.isNull() && (ch.isLetterOrNumber() || ch =='-'); i++) {
		x_++;
		ch = line_.at(x_);
	}
	if (i == 0) {
//		LOG("return from 0");
		return QString("");
	}


//	LOG("advanceWord selection %d/%d %d/%d", y_, xword_, y_, x_);
	//lucian: it might return all string!!!
	QString retval = line_.mid(xword_, x_ - xword_);
	if (retval.isNull()) {
//		LOG("return isNull");
		return QString("");
	}

//	LOG("advanceEndWord %s", (const char *)retval.utf8());
//	LOG("exit advanceEndWord with index %d", x_);
	return retval;
}


bool Editor::skeepXML() {
//	LOG("enter skeepXML with index %d", x_);
	int lines = e_->numLines();
	LOG("lines %d", lines);
	QChar ch = line_.at(x_);

	if (ch == '<') {
//		LOG("Editor::skeepXML - xml detected");
		while (1) {
			x_++;
			ch = line_.at(x_);
			if (ch == '>')
				break;
			if (ch.isNull()) {
				// advance here to a new line
				if ((y_ + 1) > lines) {
//					LOG("advanceXML end of file detected");
					e_->setCursorPosition(y_, x_, false);
					return true;
				}
				y_++;
				x_ = 0;
				line_ = e_->textLine(y_);
//				LOG("skeepXML sml advance para %d, index %d", y_, x_);
			}
		}

//		LOG("exit skeepXML with index %d", x_);
		return true;
	} else if (ch == '/' || ch == '&') {
//		LOG("Editor::skeepXML - path detected");
		x_++;
		QChar ch = line_.at(x_);
		while (ch.isLetterOrNumber()) {
			x_++;
			ch = line_.at(x_);
		}
//		LOG("exit skeepXML path with index %d", x_);
		return true;
	}


//	LOG("exit skeepXML with index %d", x_);
	return false;

}

void Editor::spellAdvance()
{
	
	if (e_->atEnd()) {
		e_->selectAll(true);
		msgScreen("end of file");
		return;
	}

//	e_->moveCursor(QTextEdit::MoveForward, true);
//	e_->moveCursor(QTextEdit::MoveBackward, true);

	e_->getCursorPosition(&y_, &x_);
	line_ = e_->textLine(y_);

	// just in case the cursor is in the middle of a word	
	advanceEndWord();

	DbDict *hptr = pdict_->getMaster();
	DbUser *uptr = pdict_->getUser();
	DbUser *iptr = pdict_->getIgnore();
	DbUser *fptr = pdict_->getFixed();
	while (1) {
		e_->selectAll(false);
		// advance to a new word
		advanceStartWord();  
		if (e_->atEnd()) {
			cleanMWords();
			msgScreen("end of file");
			return;
		}

		// skeep XML markup
		if (skeepXML())
			continue;

		//e->cursorWordForward(true);
		QString text = advanceEndWord();

		if (text.isEmpty() || text.isNull())
			continue;

		// look for word in dictionary
		Q3CString utext = text.utf8();
		const char *ctext = utext;

		LOG("text:%s", ctext);

		// text with numbers is assumed not to be a target word for spelling
		// example: hda3
		if (hasnumeric(ctext)) {
			LOG("found numeric");
			continue;
		}

		char *active_word = extract_word((char *) ctext);
		if (active_word == 0) {
			LOG("couldn't extract word");
			// lucian to fix1: we might get here because the word started with an
			// utf8 char that is not a valid word char
			// just jump over this char for now
			e_->moveCursor(Q3TextEdit::MoveForward, false);
			continue;
		}

		LOG("text active:%s", active_word);
		if (!hptr->findString(active_word) &&	// if not in dictionary
		    !fptr->findString(active_word, true) &&	 // if not in fixed/names dictionary
		    !uptr->findString(active_word) &&	// if not in user dictionary
		    !iptr->findString(active_word)) {	// if not in ignore dictionary
			Stats::instance()->notFound();
			cleanMWords();
			// skeep lookalike for words consisting in a single letter
			if (strlen(active_word) == 1) {
				delete[] active_word;
				break;
			}
			doLookalike(active_word);
			delete[] active_word;
			break;
		} else
			Stats::instance()->found();


		delete[] active_word;
	}

	e_->setCursorPosition(y_, x_, false);
	e_->setSelection(y_, xword_, y_, x_);
}


void Editor::cleanMWords() {
	mwords_->changeItem(mw1_, QString(" "));
	mwords_->changeItem(mw2_, QString(" "));
	mwords_->changeItem(mw3_, QString(" "));
	mwords_->changeItem(mw4_, QString(" "));
	mwords_->changeItem(mw5_, QString(" "));
	*word1_ = '\0';
	*word2_ = '\0';
	*word3_ = '\0';
	*word4_ = '\0';
	*word5_ = '\0';
}

void Editor::replaceWord1() {
	if (*word1_ == ' ' || *word1_ == '\0')
		return;
	LOG("replaceWord1 %s", word1_);
	char word[100];
	strcpy(word, word1_);
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	spellAdvance();
}

void Editor::replaceWord2() {
	if (*word2_ == ' ' || *word2_ == '\0')
		return;
	LOG("replaceWord2 %s", word2_);
	char word[100];
	strcpy(word, word2_);
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	spellAdvance();
}

void Editor::replaceWord3() {
	if (*word3_ == ' ' || *word3_ == '\0')
		return;
	LOG("replaceWord3 %s", word3_);
	char word[100];
	strcpy(word, word3_);
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	spellAdvance();
}

void Editor::replaceWord4() {
	if (*word4_ == ' ' || *word4_ == '\0')
		return;
	LOG("replaceWord4 %s", word4_);
	char word[100];
	strcpy(word, word4_);
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	spellAdvance();
}

void Editor::replaceWord5() {
	if (*word5_ == ' ' || *word5_ == '\0')
		return;
	LOG("replaceWord5 %s", word5_);
	char word[100];
	strcpy(word, word5_);
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	spellAdvance();
}

void Editor::spellIgnore()
{
	QString text = e_->markedText();
	Q3CString utext = text.utf8();
	const char *ctext = utext;
	char *active_word = extract_word((char *) ctext);
	if (active_word == 0) {
		active_word = extract_word((char *) (ctext + 1));
		if (active_word == 0) {
			spellAdvance();
			return;
		}
	}
//	LOG("ignore %s", active_word); 
	pdict_->getIgnore()->setString(active_word);
	delete[] active_word;

	spellAdvance();
}

void Editor::spellAdd()
{
	QString text = e_->markedText();
	Q3CString utext = text.utf8();
	const char *ctext = utext;
	char *active_word = extract_word((char *) ctext);
	if (active_word == 0) {
		active_word = extract_word((char *) (ctext + 1));
		if (active_word == 0) {
			spellAdvance();
			return;
		}
	}
//	LOG("add %s", active_word);
	pdict_->getUser()->setString(active_word);
	delete[] active_word;

	spellAdvance();
}

void Editor::spellName()
{
	QString text = e_->markedText();
	Q3CString utext = text.utf8();
	const char *ctext = utext;
	char *active_word = extract_word((char *) ctext);
	if (active_word == 0) {
		active_word = extract_word((char *) (ctext + 1));
		if (active_word == 0) {
			spellAdvance();
			return;
		}
	}
//	LOG("name %s", active_word);
	pdict_->getFixed()->setString(active_word, true);
	delete[] active_word;

	spellAdvance();
}


void Editor::textChanged()
{
//LOG("Text changed");
	if (changed_ == false) {
		QString fname = getFname();
		setFname(fname, true);
		fname = caption();
		b_.setActiveFName(fname);
		redoBufMgrList();
	}
	changed_ = TRUE;
//lucian: is coming here for each key press;
	cleanMWords();
}
void Editor::clicked(int para, int pos)
{
	LOG("clicked %d/%d", para, pos);
//lucian: is coming here for each key press;
	cleanMWords();
}

void Editor::undo() {
	cleanMWords();
	e_->undo();
}

void Editor::cut() {
	LOG("Editor cut");
	cleanMWords();
	e_->cut();
}
void Editor::copy() {
	cleanMWords();
	e_->copy();
}
void Editor::paste() {
	cleanMWords();
	e_->paste();
}
void Editor::e_delete() {
	cleanMWords();
	e_->del();
}

void Editor::selectAll() {
	cleanMWords();
	e_->selectAll(true);
}

bool Editor::advanceSearch(QString tofind, bool ignore_case) {
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	int lines = e_->numLines();

	if (para >= lines)
		return false;

	while (para < lines) {
		QString line = e_->textLine(para);
		index = line.find(tofind, index, !ignore_case);
		if (index != -1) {
			e_->setCursorPosition(para, index);
			e_->setSelection(para, index, para, index + tofind.length());
			e_->getCursorPosition(&para, &index);
			cursorPosChanged(para, index);
			e_->update();
			return true;
		}
		index = 0;
		para++;
	}
	return false;	
}


void Editor::replace() {
	cleanMWords();
	QString tofind = "";
	QString replacement = "";
	ReplaceDialog *rd = 0;

	int para;
	int index;

	while (1) {
		e_->getCursorPosition(&para, &index);
		if (rd == 0) {
			rd = new ReplaceDialog(tofind, replacement);
			rd->setModal(false);
			rd->setCaption("Replace");
			rd->show();
		}

		tofind = rd->getWordToFind();
		replacement = rd->getReplacement();
		qApp->processEvents();

		volatile ReplaceDialog::ReplaceAction raction = rd->getAction();
		if (raction == ReplaceDialog::ACTION_FINDNEXT) {
			rd->printAction();
			rd->reset();

			if (!advanceSearch(tofind)) {
				msgScreen("End of file");
				break;
			}
		}
		else if (raction == ReplaceDialog::ACTION_REPLACE) {
			rd->printAction();
			rd->reset();
			e_->del();
			e_->insertAt(replacement, para, index - tofind.length(), false);
			e_->setCursorPosition(para, index - tofind.length() + replacement.length(), false);

			if (!advanceSearch(tofind)) {
				msgScreen("End of file");
				break;
			}
		}
		else if (raction == ReplaceDialog::ACTION_REPLACEALL) {
			rd->printAction();

			while (advanceSearch(tofind)) {
				e_->getCursorPosition(&para, &index);
				e_->del();
				e_->insertAt(replacement, para, index - tofind.length(), false);
				e_->setCursorPosition(para, index - tofind.length() + replacement.length(), false);
			}

			msgScreen("End of file");
			break;
		}
		else if (raction == ReplaceDialog::ACTION_CANCEL) {
			rd->printAction();
			break;
		}
	}
	delete rd;
}


void Editor::search() {
	cleanMWords();
	SearchDialog *sd = new SearchDialog();
	sd->setCaption("Enter search word");
	if (!sd->exec()) {
		delete sd;
		return;
	}

	QString sword = sd->getWord();
	search_ = sword;
	ignore_case_ = sd->getIgnoreCase();
	delete sd;

	if (sword == "") {
		msgScreen("no word entered");
		return;
	}
	Q3CString utext = sword.utf8();
	const char *cstr = utext;
	LOG("Editor::search %s", cstr);

	if (!advanceSearch(cstr, ignore_case_)) {
		msgScreen("End of file");
	}

}

void Editor::searchAgain() {
	cleanMWords();
	if (search_ == "")
		return;
	Q3CString utext = search_.utf8();
	const char *cstr = utext;
	LOG("Editor::searchAgain %s", cstr);

	if (!advanceSearch(cstr, ignore_case_)) {
		msgScreen("End of file");
	}
}



bool Editor::extractFileAndLine(const QString &in, QString &filename, int &line) {
	filename = in.section("::", 0, 0);
	if (filename.isEmpty() || filename.isNull()) {
		LOG("filename section empty or null");
		return false;
	}
	LOG("extracted file: %s", (const char *) filename);

	QString linenum = in.section("::", 1, 1);
	LOG("linenum: %s", (const char *) linenum);
	if (linenum.isEmpty() || linenum.isNull()) {
		LOG("line section empty or null");
		return false;
	}
	line = atoi((const char *) linenum);
	if (line <= 0)
		return false;
	LOG("extracted line: %d", line);

	return true;
}

void Editor::clickedTags(int para, int pos)
{
	LOG("clickedTw %d/%d", para, pos);
	QString sline = tw_tags_->textLine(para);
	QString filename;
	int line = 0;
	if (extractFileAndLine(sline, filename, line)) {
		// we have a file and a line number
		// save curent text first
		saveCurentBuffer();
		// open the file in the editor window
		LOG("loading file from ctags %s", (const char *) filename);
		load(filename);
		e_->setCursorPosition(line - 1, 0, false);
	}
	else if (extractFileAndLine(tw_tags_->textLine(0), filename, line)) {
		// we have a file and a line number
		// save curent text first
		saveCurentBuffer();
		// open the file in the editor window
		LOG("loading file from ctags %s", (const char *) filename);
		load(filename);
		e_->setCursorPosition(line - 1, 0, false);
	}
	
	e_->setFocus();
	
}


void Editor::customEvent( QCustomEvent * e ) {
	LOG("got customEvent");
	(void) e;
	ctagsDone();
}
void Editor::ctagsDone() {
	QString msg = thread_->getMsg();
	tw_tags_->setText(msg);
	QString sline = tw_tags_->textLine(0);
	if (sline.endsWith(" single ctag definition, see below...")) {
		QString filename;
		int line = 0;
		if (extractFileAndLine(sline, filename, line))
			tw_tags_->setCursorPosition(line, 0, false);
	}
}

void Editor::roDebug() {
	QString result1 = b_.roDebug();
	QString result2 = thread_->roDebug();
	QMessageBox::warning(this,"roDebug", result1 + result2);
}

void Editor::configCtags() {
	LOG("qwerty config ctags");
	cleanMWords();
	CtagsDialog *d = new CtagsDialog();
	d->setCaption("ctags configuration");
	if (!d->exec()) {
		delete d;
		return;
	}

	Global::instance()->load_glibc_tags = d->getLibc();
	thread_->startProcessing(QString("reload tags"));

	delete d;
	return;
}

void Editor::configLanguage() {
	LOG("qwerty config language");
	cleanMWords();
	QStringList lst;
	lst << "Romanian" << "English";
	bool ok;
	int lang = 0;
	Global *glb = Global::instance();
	if (glb->lang_en)
		lang = 1;
	QString res = QInputDialog::getItem(
	        "Spelling Dictionary", "Select language:", lst, lang, TRUE, &ok,
	        this );
	if ( ok ) {
		// user selected an item and pressed OK
		if (strcmp((const char *) res, "Romanian") == 0) {
			glb->lang_ro = true;
			glb->lang_en = false;
			if (lang != 0) {
				msgScreen("Using Romanian dictionary\n");
				delete pdict_;
				pdict_ = new Dictionaries(true);
				readHomeDir(pdict_);
			}
		}
		else if (strcmp((const char *) res, "English") == 0) {
			glb->lang_ro = false;
			glb->lang_en = true;
			if (lang != 1) {
				msgScreen("Using English dictionary\n");
				delete pdict_;
				pdict_ = new Dictionaries(true);
				readHomeDir(pdict_);
			}
		}
			
	}
}
