// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************
//** $Id: qt/qwerty.h   3.3.4   edited May 27 2003 $
//**
//** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
//**
//** This file is part of an example program for Qt.  This example
//** program may be used, distributed and modified without limitation.
//**
//*****************************************************************************/

#ifndef QWERTY_H
#define QWERTY_H
#include "editor.h"

#ifdef USING_QT4
//Added by qt3to4:
#include <QCloseEvent>
#include <QResizeEvent>
#include <Q3PopupMenu>
#endif

#ifdef _WIN32
char *strcasestr(const char *haystack, const char *needle);
#endif

#include <qwidget.h>
#include <qmenubar.h>



#ifdef USING_QT3
#include <qlayout.h>
#include <qmultilineedit.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qaccel.h>
#include <qsyntaxhighlighter.h>
#endif
#ifdef USING_QT4
#include <Q3VBoxLayout>
#include <q3multilineedit.h>
#include <q3hbox.h>
#include <q3vbox.h>
#include <q3accel.h>
#include <q3syntaxhighlighter.h>
#endif

#include <qprinter.h>
#include <qdialog.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qfont.h>
#include <qlayout.h>
#include <qstatusbar.h>
#include <qtabwidget.h>


#include <stdio.h>
#include <string.h>
#include "../../config.h"
#include "editor.h"
#include "bufmgr.h"
#include "ctags.h"



// forward declarations
class Dictionaries;
class BriefEdit;
class LineAndColumn;

class Editor : public Q3VBox {
	Q_OBJECT

public:
	Editor( QWidget *parent=0, const char *name="qwerty" );
	~Editor();
	void load( const QString& fileName);
	void setDefaultFormat(FILE_FORMAT format);
	void setFname(QString fname, bool modified = false);
	QString getFname();

public slots:
	void newDoc();
	void newDocBuffer();
	void load();
	void openWwwLocation();
	void save();
	bool saveAs();
	bool saveAll();
	void print();
	void font();
	void spellAdvance();
	void spellIgnore();
	void spellAdd();
	void spellName();
	void undo();
	void cut();
	void copy();
	void paste();
	void e_delete();
	void search();
	void searchAgain();
	void replace();
	void gotoLine();
	void setBookmark();
	void gotoBookmark();
	void status();
	void uppercase();
	void lowercase();
	void dexonline();
	void unixCleanup();
	void nextBuffer();
	void closeBuffer();
	void diff();
	void sessionDiff();
	void colors();
	void configCtags();
	void configLanguage();
	void grep();
	
	// events coming here from ctag thread
	void ctagsDone();
	// customEvent is used by qt3 implementation and does a
	// function call to ctagsDone
	void customEvent( QCustomEvent * e );
	
	void selectAll();
	void toggleOverwrite();
	void recordAction();
	void playbackAction();
	void editAlias();

protected:
	void resizeEvent(QResizeEvent *);
	void closeEvent(QCloseEvent *);

private slots:
	void pickBuffer(int id);
	void textChanged();
	void clicked(int para, int pos);
	void clickedTags(int para, int pos);
	void clickedSearch(int para, int pos);
	void clickedUser(int para, int pos);
	// sepeller-sugested words
	void replaceWord1();
	void replaceWord2();
	void replaceWord3();
	void replaceWord4();
	void replaceWord5();
	void cursorPosChanged(int para, int pos);
	// bookmarks
	void setBookmark1();
	void setBookmark2();
	void setBookmark3();
	void setBookmark4();
	void setBookmark5();
	void setBookmark6();
	void setBookmark7();
	void setBookmark8();
	void setBookmark9();
	void setBookmark0();
	// user progamable keys
	void userF1();
	void userF2();
	void userF3();
	void userF4();
	// start/end of buffer
	void startOfBuffer();
	void endOfBuffer();
	// debug
	void roDebug();


private:
	void releaseAll();
	char *format2str(FILE_FORMAT format);
	void msgScreen(const char *str, int mstime = 0);
	bool saveCommon( const QString& fileName, FILE_FORMAT format, bool simple_save = false);
	FILE_FORMAT detectWwwFormat(const char *str);
	QString toUtf8(const char *str, FILE_FORMAT format);
	void advanceStartWord();
	QString advanceEndWord();
	bool skeepXML();

	void cleanMWords();
	// lookalike words entry point
	void doLookalike(char *active_word);
	// first i to â lookalike
	bool lookalike1(char *active_word, char *dest);
	// the second i to â lookalike
	bool lookalike2(char *active_word, char *dest);
	// first î to â lookalike
	bool lookalike3(char *active_word, char *dest);
	// flip char lookalike
	bool lookalike4(char *active_word, char *dest);
	// fix teutzz spelling: tz -> ţ
	bool lookalike5(char *active_word, char *dest);
	// fix sh spelling: sh -> ş
	bool lookalike6(char *active_word, char *dest);
	// insert lookalike in empty storage word
	void insertLookalike(char *str);
	// save curent buffer in bufmgr
	void saveCurentBuffer();

	char *stripDexonline(char *str);
	void redoBufMgrList();
	bool advanceSearch(QString tofind, bool ignore_case = false);
	bool extractFileAndLine(const QString &in, QString &filename, int &line);
	bool extractFileAndLine2(const QString &in, QString &filename, int &line);

	Q3Accel *a_;
	Q3PopupMenu *file_;
	Q3PopupMenu *edit_;
	Q3PopupMenu *tools_;
	Q3SyntaxHighlighter *syntax_;
//	int msgid_; // id for message in menubar
	QMenuBar       *m_;
	QMenuBar       *mwords_;
	int mw1_;	//id for the first word in mwords_ menu bar
	int mw2_;	//id for the second word in mwords_ menu bar
	int mw3_;	//id for the third word in mwords_ menu bar
	int mw4_;	//id for the fourth word in mwords_ menu bar
	int mw5_;	//id for the fifth word in mwords_ menu bar
	char word1_[100];
	char word2_[100];
	char word3_[100];
	char word4_[100];
	char word5_[100];

	BriefEdit *e_;
#ifndef QT_NO_PRINTER
	QPrinter        printer_;
#endif
	bool changed_;
	Dictionaries *pdict_;
	QString search_;
	bool ignore_case_;
	FILE_FORMAT default_format_;

	QString line_;
	int x_;
	int y_;
	int xword_;
	int yword_;

	// bookmarks
	struct Bookmark {
		int x_;
		int y_;
		QString fname_;
		Bookmark(): x_(0), y_(0), fname_("") {}
	};
	Bookmark book_[MAXBOOKMARKS];
	
	// buffer manager
	BufMgr b_;
	Q3PopupMenu *buffers_;

	// user area
	QTabWidget *tw_;
	Q3MultiLineEdit *tw_user_;
	Q3MultiLineEdit *tw_search_;
	Q3MultiLineEdit *tw_tags_;
	Q3SyntaxHighlighter *tw_tags_syntax_;

	// status bar
	QStatusBar *status_;
	LineAndColumn *lac_;

	// ctags
//#ifdef USING_QT4
	CtagsThread *thread_;
//#endif

};

#endif // QWERTY_H


