//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************

#include "qwerty.h"
#include "utils.h"
#include "dialogs.h"
#include "briefedit.h"
#include "diffdialog.h"
#include "colordialog.h"
#include "aliasdialog.h"
#include <qfileinfo.h>
#include <qmessagebox.h>
#include "qtutils.h"
#include <qfiledialog.h>
#include <qinputdialog.h>

void Editor::editAlias() {
LOG("qwerty edit alias");
	cleanMWords();
	AliasDialog *ad = new AliasDialog();
	ad->setCaption("Edit Alias");
	if (!ad->exec()) {
		delete ad;
		return;
	}

	e_->reloadAlias();
	delete ad;
	return;
}

void Editor::colors() {
	cleanMWords();
	ColorDialog *cd = new ColorDialog();
	cd->setCaption("Set colors");
	if (!cd->exec()) {
		delete cd;
		return;
	}

	Global::instance()->c_background = cd->getBackground();
	Global::instance()->c_foreground = cd->getForeground();
	Global::instance()->c_highlight = cd->getHighlight();
	Global::instance()->c_tag1 = cd->getTag1();
	Global::instance()->c_tag2 = cd->getTag2();
	Global::instance()->c_tag3 = cd->getTag3();
	Global::instance()->c_tag4 = cd->getTag4();

	e_->setPaletteBackgroundColor(Global::instance()->c_background);
	e_->setPaletteForegroundColor(Global::instance()->c_foreground);
	QPalette qp = e_->palette();
	qp.setColor(QPalette::Active, QColorGroup::Highlight, Global::instance()->c_highlight);
	qp.setColor(QPalette::Inactive, QColorGroup::Highlight, Global::instance()->c_highlight);
	e_->setPalette(qp);
	syntax_->rehighlight();

	tw_user_->setPaletteBackgroundColor(Global::instance()->c_background);
	tw_user_->setPaletteForegroundColor(Global::instance()->c_foreground);
	tw_user_->setPalette(qp);

	tw_search_->setPaletteBackgroundColor(Global::instance()->c_background);
	tw_search_->setPaletteForegroundColor(Global::instance()->c_foreground);
	tw_search_->setPalette(qp);

	tw_tags_->setPaletteBackgroundColor(Global::instance()->c_background);
	tw_tags_->setPaletteForegroundColor(Global::instance()->c_foreground);
	tw_tags_->setPalette(qp);

	update();

	delete cd;

}


void Editor::gotoLine() {
	cleanMWords();
	LineDialog *ld = new LineDialog();
	ld->setCaption("Enter line number");
	if (!ld->exec())
		return;

	int line = ld->getLine();
	LOG("Editor::gotoline %d", line);
	delete ld;

	if (line > 0) {
		int lines = e_->numLines();
		if (line > lines)
			line = lines;
		e_->setCursorPosition(line - 1, 0);
	}
}

void Editor::gotoBookmark() {
	cleanMWords();
	BookmarkDialog *bd = new BookmarkDialog();
	bd->setCaption("Enter bookmark number");
	if (!bd->exec())
		return;
	int bookmark = bd->getBookmark();
	LOG("Editor::gotoBookmark %d", bookmark);
	delete bd;

	ASSERT(bookmark < MAXBOOKMARKS);
	if (bookmark > MAXBOOKMARKS)
		return;

	// change the buffer if necessary
	QString current_fname = getFname();
	if (current_fname != book_[bookmark].fname_) {
		bool found = false;
		int i;
		for (i = 0; i < MAXBUFMGR; i++) {
			QString bfname = b_.getFName(i);
			if (bfname.isNull())
				break;
			if (bfname.startsWith("*"))
				bfname = bfname.mid(1);
			if (book_[bookmark].fname_ == bfname) {
				found = true;
				break;
			}
		}
		if (!found)
			return;
		// save current buffer data
		saveCurentBuffer();
	
		// move to the buffer specified by the bookmark
		b_.setActiveIndex(i);
	
		// restore data
		changed_ = TRUE;
		QString fname = b_.getActiveFName();
		setFname(fname, false);
		e_->setText(b_.getActiveText());	
		if (fname.startsWith("*"))
			changed_ = TRUE;
		else
			changed_ = FALSE;
		int line = b_.getActiveLine();
		int col = b_.getActiveCol();
		e_->setCursorPosition(line, col, false);

		redoBufMgrList();
	}


	int line = book_[bookmark].y_;
	int lines = e_->numLines();
	if (line > lines)
		line = lines - 1;
	e_->setCursorPosition(line, book_[bookmark].x_);
}

void Editor::setBookmark() {
	cleanMWords();
	BookmarkDialog *bd = new BookmarkDialog();
	bd->setCaption("Enter bookmark number");
	if (!bd->exec())
		return;
	int bookmark = bd->getBookmark();
	LOG("Editor::setBookmark %d", bookmark);
	delete bd;

	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	ASSERT(bookmark < MAXBOOKMARKS);
	if (bookmark > MAXBOOKMARKS)
		return;
	book_[bookmark].x_ = index;
	book_[bookmark].y_ = para;
	book_[bookmark].fname_ = getFname();
}


void Editor::setBookmark1() {
	int bookmark = 1;
	cleanMWords();
	if (bookmark > MAXBOOKMARKS)
		return;

	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	book_[bookmark].x_ = index;
	book_[bookmark].y_ = para;
	book_[bookmark].fname_ = getFname();
}

void Editor::setBookmark2() {
	int bookmark = 2;
	cleanMWords();
	if (bookmark > MAXBOOKMARKS)
		return;

	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	book_[bookmark].x_ = index;
	book_[bookmark].y_ = para;
	book_[bookmark].fname_ = getFname();
}

void Editor::setBookmark3() {
	int bookmark = 3;
	cleanMWords();
	if (bookmark > MAXBOOKMARKS)
		return;

	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	book_[bookmark].x_ = index;
	book_[bookmark].y_ = para;
	book_[bookmark].fname_ = getFname();
}

void Editor::setBookmark4() {
	int bookmark = 4;
	cleanMWords();
	if (bookmark > MAXBOOKMARKS)
		return;

	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	book_[bookmark].x_ = index;
	book_[bookmark].y_ = para;
	book_[bookmark].fname_ = getFname();
}

void Editor::setBookmark5() {
	int bookmark = 5;
	cleanMWords();
	if (bookmark > MAXBOOKMARKS)
		return;

	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	book_[bookmark].x_ = index;
	book_[bookmark].y_ = para;
	book_[bookmark].fname_ = getFname();
}


void Editor::setBookmark6() {
	int bookmark = 6;
	cleanMWords();
	if (bookmark > MAXBOOKMARKS)
		return;

	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	book_[bookmark].x_ = index;
	book_[bookmark].y_ = para;
	book_[bookmark].fname_ = getFname();
}

void Editor::setBookmark7() {
	int bookmark = 7;
	cleanMWords();
	if (bookmark > MAXBOOKMARKS)
		return;

	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	book_[bookmark].x_ = index;
	book_[bookmark].y_ = para;
	book_[bookmark].fname_ = getFname();
}

void Editor::setBookmark8() {
	int bookmark = 8;
	cleanMWords();
	if (bookmark > MAXBOOKMARKS)
		return;

	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	book_[bookmark].x_ = index;
	book_[bookmark].y_ = para;
	book_[bookmark].fname_ = getFname();
}

void Editor::setBookmark9() {
	int bookmark = 9;
	cleanMWords();
	if (bookmark > MAXBOOKMARKS)
		return;

	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	book_[bookmark].x_ = index;
	book_[bookmark].y_ = para;
	book_[bookmark].fname_ = getFname();
}

void Editor::setBookmark0() {
	int bookmark = 0;
	cleanMWords();
	if (bookmark > MAXBOOKMARKS)
		return;

	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	book_[bookmark].x_ = index;
	book_[bookmark].y_ = para;
	book_[bookmark].fname_ = getFname();
}



void Editor::userF1() {
	tw_user_->setText("");
#ifdef _WIN32
	QTUTILS::run_program(QString("c:\\rospell\\userF1.bat"), tw_user_);
#else
	QTUTILS::run_program(QString("~/.rospell/userF1.sh"), tw_user_);
#endif
	tw_->setCurrentPage(2); //lucian: hardcoded
}

void Editor::userF2() {
	tw_user_->setText("");
#ifdef _WIN32
	QTUTILS::run_program(QString("c:\\rospell\\userF2.bat"), tw_user_);
#else
	QTUTILS::run_program(QString("~/.rospell/userF2.sh"), tw_user_);
#endif
	tw_->setCurrentPage(2); //lucian: hardcoded
}

void Editor::userF3() {
	tw_user_->setText("");
#ifdef _WIN32
	QTUTILS::run_program(QString("c:\\rospell\\userF3.bat"), tw_user_);
#else
	QTUTILS::run_program(QString("~/.rospell/userF3.sh"), tw_user_);
#endif
	tw_->setCurrentPage(2); //lucian: hardcoded
}

void Editor::userF4() {
	tw_user_->setText("");
#ifdef _WIN32
	QTUTILS::run_program(QString("c:\\rospell\\userF4.bat"), tw_user_);
#else
	QTUTILS::run_program(QString("~/.rospell/userF4.sh"), tw_user_);
#endif
	tw_->setCurrentPage(2); //lucian: hardcoded
}

void Editor::startOfBuffer() {
	cleanMWords();
	e_->setCursorPosition(0, 0);
}

void Editor::endOfBuffer() {
	cleanMWords();
	int lines = e_->numLines();
	e_->setCursorPosition(lines - 1, 0);
}

void Editor::diff() {
	cleanMWords();
	DiffDialog *dd = new DiffDialog();
	dd->setCaption("Enter files");
	if (!dd->exec()) {
		delete dd;
		return;
	}

	QString file1 = dd->getFile1();
	QString file2 = dd->getFile2();
	delete dd;
LOG("diff file1: %s", (const char *) file1);
LOG("diff file2: %s", (const char *) file2);

	QFont fnt = e_->font();
	DiffWorksDialog *wd = new DiffWorksDialog(fnt, file1, file2);
	wd->setCaption("diff");
	wd->exec();
	delete wd;
}

void Editor::sessionDiff() {
	cleanMWords();
	
	// save current buffer
//	save();
	QString file1 = caption();
	if (file1.isNull() || file1.isEmpty() || file1 == "*") {
		QMessageBox::warning(0,"Warning", "Session diff not supported on new files");
		return;
	}
	save();
	file1 = caption();
	
	// get file from backup
	QFileInfo qf(file1);
	char *f2 = Global::getBackupFname((const char *) qf.fileName());
	QString file2 = QString(f2);
	delete[] f2;
LOG("session diff file1: %s", (const char *) file1);
LOG("session diff file2: %s", (const char *) file2);

	QFont fnt = e_->font();
	if (file1.isEmpty() || file1.isNull() || file2.isEmpty() || file2.isNull())
		return;
		
	DiffWorksDialog *wd = new DiffWorksDialog(fnt, file2, file1);
	wd->setCaption("session diff");
	wd->exec();
	delete wd;
}

 
void Editor::grep() {
	LOG("qwerty grep");
	cleanMWords();
	
	// check for grep presence
	if (!Global::checkWhich("grep"))
		return;

	// get grep string
 	bool ok;
	QString text = QInputDialog::getText("grep", "Enter grep string:", QLineEdit::Normal, QString::null, &ok, this );
	if ( ok && !text.isEmpty() );
	else {
		return;
	}
 		
	
	QString s = QFileDialog::getExistingDirectory("./", this, "get existing directory", "Choose a directory", TRUE );
	if (s.isNull() || s.isEmpty()) {
		return;
	}
	
	tw_search_->setText("");
#ifdef _WIN32
// not implemented	QTUTILS::run_program(QString("c:\\rospell\\userF1.bat"), tw_user_);
#else
	QTUTILS::run_program(QString("grep -n -R ") + text + QString(" ") + s, tw_search_, "\ngrep done\n");
#endif
	tw_->setCurrentPage(1); //lucian: hardcoded
}

bool Editor::extractFileAndLine2(const QString &in, QString &filename, int &line) {
	filename = in.section(":", 0, 0);
	if (filename.isEmpty() || filename.isNull()) {
		LOG("filename section empty or null");
		return false;
	}
	LOG("extracted file: %s", (const char *) filename);

	QString linenum = in.section(":", 1, 1);
	LOG("linenum: %s", (const char *) linenum);
	if (linenum.isEmpty() || linenum.isNull()) {
		LOG("line section empty or null");
		return false;
	}
	line = atoi((const char *) linenum);
	if (line <= 0)
		return false;
	LOG("extracted line: %d", line);

	return true;
}

void Editor::clickedSearch(int para, int pos)
{
	LOG("clickedSearch %d/%d", para, pos);
	QString sline = tw_search_->textLine(para);
	QString filename;
	int line = 0;
	if (extractFileAndLine2(sline, filename, line)) {
		// we have a file and a line number
		// save curent text first
		saveCurentBuffer();
		// open the file in the editor window
		LOG("loading file from search %s", (const char *) filename);
		load(filename);
		e_->setCursorPosition(line - 1, 0, false);
		e_->setFocus();
	}
}

void Editor::clickedUser(int para, int pos)
{
	LOG("clickedUser %d/%d", para, pos);
	QString sline = tw_user_->textLine(para);
	QString filename;
	int line = 0;
	if (extractFileAndLine2(sline, filename, line)) {
		// we have a file and a line number
		// save curent text first
		saveCurentBuffer();
		// open the file in the editor window
		LOG("loading file from search %s", (const char *) filename);
		load(filename);
		e_->setCursorPosition(line - 1, 0, false);
		e_->setFocus();
	}
}
