//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************

#include "qwerty.h"
#include "spell.h"

void Editor::insertLookalike(char *str) {
	ASSERT(str != 0);

	if (*word1_ == '\0') {
		strcpy(word1_, str);
		mwords_->changeItem(mw1_, QString::fromUtf8(word1_));
		return;
	}
	if (*word2_ == '\0') {
		strcpy(word2_, str);
		mwords_->changeItem(mw2_, QString::fromUtf8(word2_));
		return;
	}
	if (*word3_ == '\0') {
		strcpy(word3_, str);
		mwords_->changeItem(mw3_, QString::fromUtf8(word3_));
		return;
	}
	if (*word4_ == '\0') {
		strcpy(word4_, str);
		mwords_->changeItem(mw4_, QString::fromUtf8(word4_));
		return;
	}
	if (*word5_ == '\0') {
		strcpy(word5_, str);
		mwords_->changeItem(mw5_, QString::fromUtf8(word5_));
		return;
	}
}


void Editor::doLookalike(char *active_word) {

	DbDict *hptr = pdict_->getMaster();

	// grab any valid words form dict based on active_words
	hptr->findLookalike(active_word, word1_, word2_, word3_, word4_, word5_);
	mwords_->changeItem(mw1_, QString::fromUtf8(word1_));
	mwords_->changeItem(mw2_, QString::fromUtf8(word2_));
	mwords_->changeItem(mw3_, QString::fromUtf8(word3_));
	mwords_->changeItem(mw4_, QString::fromUtf8(word4_));
	mwords_->changeItem(mw5_, QString::fromUtf8(word5_));

	// look for words with i to â rule
	char tmp_word[100];
	bool look_done = lookalike1(active_word, tmp_word);
	if (look_done) {
		LOG("found lookalike1");
		insertLookalike(tmp_word);
	}

	// look for the second apparition in words with i to â rule
	look_done = lookalike2(active_word, tmp_word);
	if (look_done) {
		LOG("found lookalike2");
		insertLookalike(tmp_word);
     }

	// look for the first apparition in words with î to â rule
	look_done = lookalike3(active_word, tmp_word);
	if (look_done) {
		LOG("found lookalike3");
		insertLookalike(tmp_word);
     }

	// flip char lookalike
	look_done = lookalike4(active_word, tmp_word);
	if (look_done) {
		LOG("found lookalike4");
		insertLookalike(tmp_word);
	}

	// fix teutzz spelling: tz -> ţ
	look_done = lookalike5(active_word, tmp_word);
	if (look_done) {
		LOG("found lookalike5");
		insertLookalike(tmp_word);
	}

	// fix sh spelling: sh -> ş
	look_done = lookalike6(active_word, tmp_word);
	if (look_done) {
		LOG("found lookalike5");
		insertLookalike(tmp_word);
	}

	LOG("word1:%s", word1_);
	LOG("word2:%s", word2_);
	LOG("word3:%s", word3_);
	LOG("word4:%s", word4_);
	LOG("word5:%s", word5_);
}

// flip char lookalike
//lucian: does not suport flipping romanian utf chars yet
bool Editor::lookalike4(char *active_word, char *dest) {
	ASSERT(active_word != 0);
	ASSERT(dest != 0);
	char newword[100 + 1];
	strncpy(newword, active_word, 100);
	unsigned char *ptr = (unsigned char *) newword;
	DbDict *hptr = pdict_->getMaster();

	while (*ptr != 0 && *(ptr + 1) != 0) {
		if (!(*ptr & 0x80  || *(ptr + 1) & 0x80)) {

			char tmp = *ptr;
			*ptr = *(ptr + 1);
			*(ptr + 1) = tmp;
			if (hptr->findString(newword)) {
				strcpy(dest, newword);
				return true;
			}

			tmp = *ptr;
			*ptr = *(ptr + 1);
			*(ptr + 1) = tmp;
		}
		if (*ptr & 0x80)
			ptr += 2;
		else
			ptr++;
	}
	return false;
}

// first i to â lookalike
bool Editor::lookalike1(char *active_word, char *dest) {
	ASSERT(active_word != 0);
	ASSERT(dest != 0);
	char *ptr;
	if ((ptr = strstr(active_word, "i")) != 0) {
		strcpy(dest, active_word);
		ptr = strstr(dest, "i");
		ASSERT(ptr != 0);
		if (ptr == 0)
			return false; // just in case
		char ptrtmp[100];
		strcpy(ptrtmp, "â");
		strcat(ptrtmp, ptr + 1);
		strcpy(ptr, ptrtmp);
		// look for the new word in dictionary
		DbDict *hptr = pdict_->getMaster();
		if (hptr->findString(dest))
			return true;
		else
			*dest = '\0';
	}
	return false;
}

// second i to â lookalike
bool Editor::lookalike2(char *active_word, char *dest) {
	ASSERT(active_word != 0);
	ASSERT(dest != 0);
	char *ptr;
	if ((ptr = strstr(active_word, "i")) != 0) {
		if ((ptr = strstr(ptr + 1, "i")) != 0) {
			strcpy(dest, active_word);
			ptr = strstr(dest, "i");
			ASSERT(ptr != 0);
			if (ptr == 0)
				return false; // just in case
			ptr = strstr(ptr + 1, "i");
			ASSERT(ptr != 0);
			if (ptr == 0)
				return false; // just in case
			char ptrtmp[100];
			strcpy(ptrtmp, "â");
			strcat(ptrtmp, ptr + 1);
			strcpy(ptr, ptrtmp);
			// look for the new word in dictionary
			DbDict *hptr = pdict_->getMaster();
			if (hptr->findString(dest))
				return true;
			else
				*dest = '\0';
		}
	}
	return false;
}

// first î to â lookalike
bool Editor::lookalike3(char *active_word, char *dest) {
	ASSERT(active_word != 0);
	ASSERT(dest != 0);
	char *ptr;
	if ((ptr = strstr(active_word, "î")) != 0) {
		strcpy(dest, active_word);
		ptr = strstr(dest, "î");
		ASSERT(ptr != 0);
		if (ptr == 0)
			return false; // just in case
		char ptrtmp[100];
		strcpy(ptrtmp, "â");
		strcat(ptrtmp, ptr + 2);
		strcpy(ptr, ptrtmp);
		// look for the new word in dictionary
		DbDict *hptr = pdict_->getMaster();
		if (hptr->findString(dest))
			return true;
		else
			*dest = '\0';
	}
	return false;
}

bool Editor::lookalike5(char *active_word, char *dest) {
	ASSERT(active_word != 0);
	ASSERT(dest != 0);
	char *ptr;
	if ((ptr = strstr(active_word, "tz")) != 0) {
		strcpy(dest, active_word);
		ptr = strstr(dest, "tz");
		ASSERT(ptr != 0);
		if (ptr == 0)
			return false; // just in case
		char ptrtmp[100];
		strcpy(ptrtmp, "ţ");
		strcat(ptrtmp, ptr + 2);
		strcpy(ptr, ptrtmp);
		// look for the new word in dictionary
		DbDict *hptr = pdict_->getMaster();
		if (hptr->findString(dest))
			return true;
		else
			*dest = '\0';
	}
	return false;
}

bool Editor::lookalike6(char *active_word, char *dest) {
	ASSERT(active_word != 0);
	ASSERT(dest != 0);
	char *ptr;
	if ((ptr = strstr(active_word, "sh")) != 0) {
		strcpy(dest, active_word);
		ptr = strstr(dest, "sh");
		ASSERT(ptr != 0);
		if (ptr == 0)
			return false; // just in case
		char ptrtmp[100];
		strcpy(ptrtmp, "ş");
		strcat(ptrtmp, ptr + 2);
		strcpy(ptr, ptrtmp);
		// look for the new word in dictionary
		DbDict *hptr = pdict_->getMaster();
		if (hptr->findString(dest))
			return true;
		else
			*dest = '\0';
	}
	return false;
}

